#include "netrun.h"

/*
 * ]-*-[ bank.c
 *
 * This file contains all the functions dealing with the bank. Deposit,
 * withdraw, process (stolen :) ) cheques, get a loan.
 *
 * UNFIN Unpaid loans aren't yet penalised. I'm guessing very complex stuff, or something during the maint.
 * UNFIN Processing fees for loan.
 */

/*
 * Main bank function.
 */

void do_bank(RUNNER_DATA * pRunner)
{
    bool repeat;
    char *choice;

    repeat = TRUE;
    while (repeat)
    {
        if (where != W_STREET)
            return;

        running_off();

        menu_init();
        menu("", "A street ATM");
        menu("C", "Deposit a cheque");
        menu("D", "Make a deposit");
        menu("W", "Make a withdrawal");
        menu(".", "Deposit ALL credits");
        menu("/", "Withdraw ALL credits");
        if (pRunner->loan == 0)
            menu("L", "Request a loan");
        else
            menu("L", "Repay your loan");
        menu("X", "Back out");
        menu_print();
        stat(" * Credits: %ldcr Deposit: %ldcr Loan: %ldcr",
            pRunner->credits,
            pRunner->deposit,
            pRunner->loan);

        choice = input("Now what?");

        if (!strcmp(choice, "C"))
        {
            do_bank_cheque(pRunner);
        }
        else if (!strcmp(choice, "D"))
        {
            do_bank_deposit(pRunner);
        }
        else if (!strcmp(choice, "W"))
        {
            do_bank_withdraw(pRunner);
        }
        else if (!strcmp(choice, "."))
        {
            /* Deposit all credits. */
            int deposit;

            deposit = pRunner->credits;

            if (deposit > 0)
            {
                pRunner->deposit += deposit;
                pRunner->credits -= deposit;
                event("Deposited %d credits. Have a nice day.", deposit);
            }
            else
                err("You have no credits to deposit.");
        }
        else if (!strcmp(choice, "/"))
        {
            /* Withdraw all credits. */
            int withdrawal;

            withdrawal = pRunner->deposit;

            if (withdrawal)
            {
                pRunner->deposit -= withdrawal;
                pRunner->credits += withdrawal;
                event("Withdrew %d credits. Have a nice day.", withdrawal);
            }
            else
                err("You have no credits to withdraw.");
        }
        else if (!strcmp(choice, "L"))
        {
            do_bank_loan(pRunner);
        }
        else if (!strcmp(choice, "X"))
        {
            repeat = FALSE;
        }
        else
        {
            err("Do what?");
        }
    }
}

/*
 * Deposit a cheque (particular file type) in the bank.
 *
 * UNFIN Randomise the spoils from cheques somewhat. Sometimes they're a bit less, sometimes they bounce, sometimes they're booby-trapped. :) Perhaps a transaction fee, too? 
 */

void do_bank_cheque(RUNNER_DATA * pRunner)
{
    bool repeat;
    PROGRAM_DATA *pProgram;

    if (where != W_STREET)
        return;

    running_off();

    if (pRunner->stored == NULL)
    {
        err("You have no cheque files to deposit.");
        return;
    }

    repeat = TRUE;
    while (repeat)
    {
        if (where != W_STREET)
            return;

        stat(" * Credits: %ldcr Deposit: %ldcr Loan: %ldcr",
            pRunner->credits,
            pRunner->deposit,
            pRunner->loan);

        pProgram = input_stored_program(pRunner, "Deposit which cheque file?");

        if (pProgram == NULL)
        {
            return;
        }

        if (pProgram->type == P_CRED)
        {
            if (yn("Deposit this cheque file?"))
            {
                int deposit;

                deposit = server_spoils_table[pProgram->str];
                pRunner->deposit += deposit;
                kill_program(pProgram);
                event("Deposited %d credits from cheque. Have a nice day.",
                    deposit);
            }
        }
        else
        {
            err("That's not a cheque file.");
        }
    }

}

/*
 * Deposit 'raw' credits.
 */

void do_bank_deposit(RUNNER_DATA * pRunner)
{
    int deposit;

    if (where != W_STREET)
        return;

    running_off();

    if (pRunner->credits < 1)
    {
        err("You have no loose credits to deposit.");
        return;
    }

    /* No number checks - all in input_num. Aren't subroutines good? :) */
    deposit = input_num(1, pRunner->credits, "Deposit how much?");

    if (deposit > 0)
    {
        pRunner->credits -= deposit;
        pRunner->deposit += deposit;
        event("Deposited %d credits. Have a nice day.", deposit);
    }
}

/*
 * Withdraw credits.
 */

void do_bank_withdraw(RUNNER_DATA * pRunner)
{
    int withdrawal;

    if (where != W_STREET)
        return;

    running_off();

    if (pRunner->deposit < 1)
    {
        err("You have no balance to withdraw.");
        return;
    }

    withdrawal = input_num(1, pRunner->deposit, "Withdraw how much?");

    if (withdrawal > 0)
    {
        pRunner->deposit -= withdrawal;
        pRunner->credits += withdrawal;
        event("Withdrew %d credits. Have a nice day.", withdrawal);
    }

}

/*
 * Loan processing - get one, or pay one off.
 *
 * HACK UNFIN Penalties for defaulting on loans unfinished! No idea how to do them, either.
 * UNFIN Processing fees for loan.
 */

void do_bank_loan(RUNNER_DATA * pRunner)
{
    if (where != W_STREET)
        return;

    running_off();

    /* Not got a loan? */
    if (pRunner->loan <= 0)
    {
        /*
         * Requesting a loan
         */

        int loan;

        menu_init();
        menu("", "Loan terms:");
        menu("", "The interest is %d%%, daily.", LOAN_INTEREST);
        menu("", "You may borrow up to %d%% of your networth.", LOAN_MAXWORTH);
        menu("", "You may only have one loan at a time.");
        menu("", "Repayments are not enforced, but [unfin].");
        menu_print();

        loan = input_num(1, (networth(pRunner) * LOAN_MAXWORTH / 100), "Borrow how much?");

        if (loan < 1)
        {
            return;
        }

        pRunner->loan += loan;
        pRunner->deposit += loan;

        event("Loan of %dcr deposited in account. Have a nice day.",
            loan);

        return;
    }
    else
    {
        /*
         * Paying back a loan
         */

        int repayment;
        bool repeat;

        repeat = TRUE;
        while (repeat)
        {
            repeat = FALSE;
            repayment = input_num(1, pRunner->loan, "Pay back how much of your loan?");

            if (repayment < 1)
            {
                return;
            }

            if (repayment > pRunner->credits)
            {
                err("You don't have that many credits in hand to pay off your loan with.");
                repeat = TRUE;
            }
            else
            {
                pRunner->loan -= repayment;
                pRunner->credits -= repayment;

                event("Loan repayment of %dcr repaid. %dcr remains. Have a nice day.",
                    repayment, pRunner->loan);

                return;
            }
        }
    }
}

/*
 * Returns runner's networth. See inside for method of calculation.
 *
 * HACK This function is rather nasty in that it often must be updated when
 * a new game feature that involves credits is added, for most of those would change networth.
 */
int networth(RUNNER_DATA * pRunner)
{
    int worth;
    PROGRAM_DATA *pProgram;
    int i;

    worth = 0;

    /*
     * The basics.
     */

    worth += pRunner->credits;
    worth += pRunner->deposit;
    worth -= pRunner->loan;

    /* 
     * Programs
     */

    pProgram = pRunner->programs;

    while (pProgram != NULL)
    {
        if (pProgram->str > 0)
        {
            worth += pProgram->cost;

            /* Perhaps jack cost down for virus? */
        }

        pProgram = pProgram->next;
    }

    pProgram = pRunner->stored;

    while (pProgram != NULL)
    {
        if (pProgram->str > 0)
        {
            worth += pProgram->cost;

            /* Perhaps jack cost down for virus? */
        }

        pProgram = pProgram->next;
    }

    /*
     * Rank, armor 
     */

    for (i = 0; i < pRunner->rank; i++)
    {
        worth += rank_cost_table[i];
    }

    for (i = 1; i <= pRunner->armor; i++)
    {
        worth += armor_cost_table[i];
    }

    /* Anything else? You tell me. */

    return worth;
}
