#include "netrun.h"

/*
 * ]-*-[ core.c
 *
 * This file contains most of the out-of-running stuff - information sales,
 * deck upgrades, program and utility trading, etc. It also, for the moment,
 * contains the code that lets the player jack in.
 */

/*
 * A `sub-heading' for stuff done on the Street - buying and selling, as
 * well as healing and rank upgrades.
 */
 
void do_street(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;
    
    repeat = TRUE;
    while (repeat)
    {
        /* If we are meant to be jacked in, this allows main() to switch to the 'running loop. */
        if (where != W_STREET)
            return;
            
        running_off();
    
        menu_init();
        menu("P","Buy programs");
        menu("S","Sell programs");
        menu("I","Sell information");
        menu("D","Visit the local 'deck technician");
        menu("U","Buy and sell utilities.");
        menu("B","Visit an Autobank.");
        menu("R","Cruise the city, visiting fixers, popular people and skill-masters, to upgrade your Rank.");
        menu("H","Visit Night City General Hospital to cure some damage.");
        menu("V","View your stats");
        menu("G","Go home");
        menu_print();
        stat(" * Rank: %s Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d",
            rank_name_table[pRunner->rank],
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT);
    
        choice = input("Now what?");
        
        if (!strcmp(choice, "P"))
        {
            do_programs_buy(pRunner);
        }
        else if (!strcmp(choice, "S"))
        {
            do_programs_sell(pRunner);
        }
        else if (!strcmp(choice, "I"))
        {
            do_sellinfo(pRunner);
        }
        else if (!strcmp(choice, "D"))
        {
            do_deck(pRunner);
        }
        else if (!strcmp(choice, "U"))
        {
            do_utils(pRunner);
        }
        else if (!strcmp(choice, "B"))
        {
            do_bank(pRunner);
        }
        else if (!strcmp(choice, "R"))
        {
            do_day_rank(pRunner);
        }
        else if (!strcmp(choice, "H"))
        {
            do_day_heal(pRunner);
        }
        else if (!strcmp(choice,"V"))
        {
            display_player(pRunner);
            morewait();
        }
        else if (!strcmp(choice, "G"))
        {
            where = W_HOME;
            return;
        }
        else
        {
            err("Do what?");
        }
    }
}

/*
 * Subheading for stuff done at home - jack in, pull programs in and out of deck,
 * go to sleep (quit), save game (TODO Decent in-character term for this).
 */
 
void do_home(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;
    
    repeat = TRUE;
    while (repeat)
    {
        /* HACK so, if we jack in, this allows main() to switch to the 'running loop. */
        if (where != W_HOME)
            return;
    
        running_off();
    
        menu_init();
        menu("J","Jack in");
        menu("I","Insert programs to your cyberdeck");
        menu("R","Remove programs from your cyberdeck");
        menu("V","View your stats");
        menu("Q","Turn in for the night");
        menu("G","Go to the Street");
        menu("D","Update your diary");
        menu_print();
        stat(
            " * Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d Armor: %d",
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT,
            pRunner->armor);
    
        choice = input("Now what?");
        
        if (!strcmp(choice, "J"))
        {
            do_startrun(pRunner);
        }
        else if (!strcmp(choice,"V"))
        {
            display_player(pRunner);
            morewait();
        }
        else if (!strcmp(choice,"I"))
        {
            do_other_insert(pRunner);
        }
        else if (!strcmp(choice,"R"))
        {
            do_other_remove(pRunner);
        }
        else if (!strcmp(choice, "G"))
        {
            where = W_STREET;
            return;
        }
        else if (!strcmp(choice, "Q"))
        {
            if (yn("Are you sure you want to quit?"))
            {
                event("You retire to your %s for the night.",
                    house_name_table[pRunner->rank]);
                where = W_END;
            }
        }
        else if (!strcmp(choice, "D"))
        {
            save_game();
        }
        else
        {
            err("Do what?");
        }
    }
}

/*
 * Sell INFO programs.
 */
 
void do_sellinfo(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    bool repeat;
    
    if (pRunner->stored == NULL)
    {
        err("You have no information to sell.");
        return;
    }

    repeat = TRUE;
    while(repeat)
    {            
        if (where != W_STREET)
            return;
    
        stat(" * Rank: %s Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d",
            rank_name_table[pRunner->rank],
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT);
        pProgram = input_stored_program(pRunner, "Sell which information?");        

        if(pProgram == NULL)
        {
            return;
        }
        
        if (pProgram->type == P_INFO)    
        {
            if (yn("Try and sell some possibly interesting information?"))
            {
                int cost = server_spoils_table[pProgram->str];
                int j = one_in_(3);
            
                event("You cruise around the Street, trying to sell some possibly valuable information.\n");
            
                switch(dice(2,6))
                {
                    case 2:
                        cost *= 0.5;
                        cost = (0 - cost);
                        if (j == 1)
                            event("It was so dangerous, the fixer demanded money for even being told about it!");
                        else if (j == 2)
                            event("You lost so much after selling it, you lost money!");    
                        else if (j == 3)
                            event("You spent so much effort trying to sell it, you lost money!");                                
                        break;
                    case 3:
                        cost = 0;
                        if (j == 1)
                            event("It was so dangerous, you couldn't find a buyer.");
                        else if (j == 2)
                            event("You lost so much after selling it, you made nothing.");    
                        else if (j == 3)
                            event("You spent so much effort trying to sell it, you made nothing.");                                
                        break;
                    case 4:
                        cost *= 0.5;
                        event("It was less valuable than you expected.");
                        break;
                    default:
                        bug("sell_prog: selling info - odd dice return\n");
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                    case 9:
                        break;
                    case 10:
                        cost *= 2;
                        event("It was more valuable than you expected.");
                        break;
                    case 11:
                        cost *= 3;
                        event("It was much more valuable than you expected!");
                        break;
                    case 12:
                        cost *= 5;
                        event("This has got to be the _best_ sale of the month!");
                        break;
                }
                
                if (cost < 0)
                {
                    if (0 - cost > pRunner->credits)
                    {
                        cost = 0 - (pRunner->credits);
                    }
                
                    event("You _lost_ %dcr on the sale.", (0 - cost));
                    
                    pRunner->credits -= (0 - cost);
                }
                else if (cost == 0)
                {
                    event("You made no money on the sale.");
                }
                else
                {
                    event("You made %dcr on the sale.", cost);                        
                    pRunner->credits += cost;
                }

                kill_program(pProgram);
            }
        }
        else
        {
            err("That's not saleable information.");
        }            
    }    
}

/*
 * Buying programs
 */

void do_programs_buy(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;

    repeat = TRUE;
    while(repeat)
    {    
        if (where != W_STREET)
            return;
    
        menu_init();
        menu("", "Buy programs:");
        menu("A","Anti-Program");
        menu("I","Intrustion");
        menu("D","Decryption");
        menu("P","Protection");
        menu("S","Stealth");
        menu("X","Go back");
        menu_print();
        stat(
            " * Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d",
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT);
             
        choice = input("Buy what type of program?");
        
        if (!strcmp(choice,"A"))
            do_programs_buy_aux(pRunner,P_ANTI_P);
        else if (!strcmp(choice,"I"))
            do_programs_buy_aux(pRunner,P_INTR);
        else if (!strcmp(choice,"D"))
            do_programs_buy_aux(pRunner,P_DECR);
        else if (!strcmp(choice,"P"))
            do_programs_buy_aux(pRunner,P_PROT);
        else if (!strcmp(choice,"S"))
            do_programs_buy_aux(pRunner,P_STEALTH);
        else if(!strcmp(choice,"X"))
            repeat = FALSE;
        else
            err("Do what?");        
    }
}

/* a HACK, this entire function is. */
void do_programs_buy_aux(RUNNER_DATA * pRunner, program_type type)
{
    int i, j, cost, choice, ch;
    bool repeat;

    repeat = TRUE;
      
    while (repeat)
    {    
        if (where != W_STREET)
            return;
        
        running_off();
    
        menu_init();
        menu("","Buy which program?");
        
        i = 0;
        j = 0;
        
        while (program_list[i].name != NULL)
        {            
            if (program_list[i].fulltype == type)
            {        
                char buf[200];
                char buf2[200];

                sprintf(buf,"%-13s %dcr",
                    program_list[i].name,
                    program_list[i].cost);

                j++;

                sprintf(buf2,"%d",j);

                menu(buf2,buf);
            }

            i++;
        }        
        
        menu("X","Back out");
        menu_print();
        stat(
            " * Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d",
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT);
        
        ch = input_num(1, j, "Buy which program?");        
        
        if (ch < 1)
        {
            return;
        }

        /* Make that an 'absolute' reference. */
        /* HACK Does this bit stink? Does to me. */
        i = 0;
        j = 0;
        choice = 0;
        
        while (program_list[i].name != NULL)
        {            
            if (program_list[i].fulltype == type)
            {                        
                j++;

                if (j == ch)
                {
                    choice = i;
                    break;
                }
            }

            i++;
        }        

        if (choice == 0)
        {
            return;
        }

        cost = program_list[choice].cost;
        
        if (program_list[choice].cost > pRunner->credits)
        {
            err("A %s costs %d cr, which you can't afford.",
                program_list[choice].name,
                program_list[choice].cost);
        }        
        else if(yn("A %s costs %dcr. Buy it?",
            program_list[choice].name,
            program_list[choice].cost))
        {
                PROGRAM_DATA * pProgram;

                pProgram = create_program((PROGRAM_MASTER *)&program_list[choice]);

                event("You buy a %s for %dcr, which you store.\n",
                    pProgram->name,
                    pProgram->cost);

                pRunner->credits -= pProgram->cost;
                program_to_stored(pRunner, pProgram);    

                /* HACK it kept on killing bought programs */
                /* It's not ours anymore. Let go! */
                pProgram = NULL;
        }                            
    }
}


void do_programs_sell(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    bool repeat;

    if ((pRunner->programs == NULL) && (pRunner->stored == NULL))
    {
        err("You have no programs to sell.");
        return;
    }
  
    repeat = TRUE;
    while(repeat)
    {
        if (where != W_STREET)
            return;
        
        running_off();
    
        stat(" * Rank: %s Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d",
            rank_name_table[pRunner->rank],
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT);
        pProgram = input_stored_program(pRunner, "Sell which program?");

        if(pProgram == NULL)
        {
            return;
        } 
        
        if (!DATA(pProgram))    
        {
            if (pProgram->str > 0)
            {
                int cost = pProgram->cost / 2;
            
                if (yn("Sell %s for %dcr?",
                    pProgram->name,
                    cost))
                {
                    event("You sell %s for %dcr.",
                        pProgram->name,
                        cost);
                    pRunner->credits += cost;
                    kill_program(pProgram);
                }
            }
            else
            {
                err("That program is damaged.");
            }
        }
        else
        {
            err("That's not a program.");
        }        
    }    
}

void do_deck(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;

    repeat = TRUE;

    while(repeat)
    {
        if (where != W_STREET)
            return;
        
        running_off();
    
        menu_init();
        menu("", "What now?");
        if (pRunner->armor < MAX_ARMOR)
            menu("U","Upgrade cyberdeck's Armor (to %d, %dcr)",
                pRunner->armor + 1, armor_cost_table[pRunner->armor + 1]);
        else
            menu("U","Upgrade cyberdeck's Armor (maxed out)");
        if (pRunner->armor > 0)
            menu("D","Downgrade cyberdeck Armor (downgrade to %d, gain %dcr)",
                pRunner->armor - 1, armor_cost_table[pRunner->armor - 1] / 2);
        else
            menu("D","Downgrade cyberdeck Armor (no armor)");        
        menu("R","Repair Rom");    
        menu("X","Go back");
        menu_print();
        stat(" * Cash: %ldcr ROM: %1d/%1d Meat: %1d/%1d Armor: %d",
            pRunner->credits,
            pRunner->rom, MAX_ROM,
            pRunner->meat, MAX_MEAT,
            pRunner->armor);
        
        
        choice = input("Now what?");
        
        if(!strcmp(choice,"U"))        
            do_deck_up(pRunner);
        else if (!strcmp(choice,"D"))
            do_deck_down(pRunner);    
        else if (!strcmp(choice,"R"))
            do_deck_rom(pRunner);    
        else if (!strcmp(choice,"X"))
            repeat = FALSE;
        else
            err("Do what?");
    }        
}

void do_deck_up(RUNNER_DATA * pRunner)
{
    if (where != W_STREET)
        return;

    if (pRunner->armor >= MAX_ARMOR)
    {
        err("Your cyberdeck's Armor is as good as it's going to get.");
        return;
    }

    if (armor_cost_table[pRunner->armor + 1] > pRunner->credits)    
    {
        err("Upgrading your cyberdeck's Armor to %d costs %dcr, which you can't afford.",
            pRunner->armor + 1, armor_cost_table[pRunner->armor + 1]);
        return;
    }
        
    if(yn("Upgrade your cyberdeck's Armor to %d for %dcr?",
        pRunner->armor + 1, armor_cost_table[pRunner->armor + 1]))
    {
        pRunner->armor++;
        pRunner->credits -= armor_cost_table[pRunner->armor];
        event("A cyberdeck technician upgrades your cyberdeck's Armor to %d for %dcr.",
            pRunner->armor,
            armor_cost_table[pRunner->armor]);
    }
}

void do_deck_down(RUNNER_DATA * pRunner)
{
    if (where != W_STREET)
        return;

    if (pRunner->armor <= 0)
    {
        err("Your cyberdeck has no Armor to sell.");
        return;
    }

    if(yn("Downgrade your cyberdeck's Armor to %d for %dcr?",
        pRunner->armor - 1, armor_cost_table[pRunner->armor] / 2))
    {
        pRunner->armor--;
        pRunner->credits += armor_cost_table[pRunner->armor + 1] / 2;
        event("A cyberdeck technician buys your cyberdeck's Armor to %d, and pays %dcr for it.",
            pRunner->armor,
            armor_cost_table[pRunner->armor + 1] / 2);
    }
}            


/*
 * Repairing deck ROM.
 */

/* UNFIN 'autoliquidate' for ROM repairs not finished. */
/*
 * Just be glad that the autoliquidate whatever only needs to go in the
 * 'double-cost for full repairs' part of the function.
 */

void do_deck_rom(RUNNER_DATA * pRunner)
{
    int cost;

    if (where != W_STREET)
        return;

    if (pRunner->rom >= MAX_ROM)
    {
        err("Your deck's ROM is not damaged.");
        return;
    }
    
    if (pRunner->rom <= 0)
    {
        cost = MAX_ROM * COST_ROM * 2;
        
        if (pRunner->credits < cost)
        {
            err("Replacing your deck's ROM will cost you %dcr, which you can't afford.",
                cost);
            return;   
        }
        else
        {
            if(yn("Replacing your deck's ROM will cost you %dcr. Do it?",
                cost))
            {
                pRunner->credits -= cost;
                pRunner->rom = MAX_ROM;
                event("A cyberdeck technician replaces your deck's ROM for %dcr.",
                    cost);
            }
            return;
        }
    }
    else
    {
        int damage, afford, repair;
                
        afford = pRunner->credits / COST_ROM;
        
        if (afford < 1)
        {
            err("Repairing your deck's ROM will cost at least %dcr, which you can't afford.",
                COST_ROM);
        }

        damage = MAX_ROM - pRunner->rom;
        
        repair = input_num(1, UMIN(damage, afford),
            "Your deck has lost %d ROM points. You can afford to repair up to %d. Repair how many?",
            damage,afford);
            
        if (repair < 1)
        {
            return;
        }
        
        cost = repair * COST_ROM;
        
        if (yn("That will cost you %dcr. Do it?", cost))
        {
            pRunner->credits -= cost;
            pRunner->rom += repair; 
            event("A cyberdeck technician repairs %d points of your deck's ROM for %dcr.",
                repair, cost);
        }
        return;    
    }    
}

void do_day_rank(RUNNER_DATA * pRunner)
{
    if (where != W_STREET)
        return;

    if (pRunner->rank >= MAX_RANK)
    {
        err("You are highly reknowned and skilled as you are going to get.");
        return;
    }

    if (pRunner->credits < rank_cost_table[pRunner->rank + 1])
    {
        err("Becoming %s will cost you %dcr, which you don't have.",
            rank_name_table[pRunner->rank + 1],
            rank_cost_table[pRunner->rank + 1]);
        return;    
    }
    
    if(yn("Going from a %s to a %s will cost you %dcr. Do it?",
            rank_name_table[pRunner->rank],
            rank_name_table[pRunner->rank + 1],
            rank_cost_table[pRunner->rank + 1]))
    {
        pRunner->credits -= rank_cost_table[pRunner->rank + 1];
        pRunner->rank++;
        
        /* Whew! */
        event("You travel the streets, and see people.\n");
        event("You visit the nightclubs, spread money around, and make friends.\n");
        event("You come to the masters of the various 'arts' of the City, and learn much.\n");
        event("You go to a number of corporations, and purchase a number of things.\n");
        event("You wander from fixer to fixer, and make contacts.\n");
        event("You do many other things.\n");
        event("You eventually come to your new %s, exhausted, %s.\n",
            house_name_table[pRunner->rank],
            car_name_table[pRunner->rank]);
        event("You are now a %s, rank %d.",
            rank_name_table[pRunner->rank],
            pRunner->rank);
        /* HACK What about that maint thing...? */
        next_day(pRunner);    
    }        
}

void do_day_heal(RUNNER_DATA * pRunner)
{
    if (where != W_STREET)
        return;

    if (pRunner->meat >= MAX_MEAT)
    {
        err("You aren't hurt.");
        return;
    }

    if (pRunner->credits < COST_HEAL)
    {
        err("A day's stay in Night City General, healing one meat point, will cost you %dcr, which you don't have.",
              COST_HEAL);
        return;    
    }
    
    if(yn("A day's stay in Night City General, healing one meat point, will cost you %dcr. Do it?",
        COST_HEAL))
    {
        pRunner->credits -= COST_HEAL;
        pRunner->meat++;

        event("You check into Night City General Hospital.\n");
        event("A doctor makes a few stitches.\n");
        event("You spend the rest of the day in a bed in a %s ward, on a %s drip feed.\n",
            (pRunner->rank > (MAX_RANK / 2)) ? "private" : "public",
            (pRunner->rank > (MAX_RANK / 2)) ? "Speedheal" : "glucose");
        event("You are checked out at night and head on to your %s.\n",
            house_name_table[pRunner->rank]);
        event("You now have %d/%d meat points.",
            pRunner->meat,
            MAX_MEAT);
        /* HACK What about that maint thing...? */
        next_day(pRunner);
    }        
}


void do_startrun(RUNNER_DATA * pRunner)
{
    int i, level;
    int servers[MAX_LEVEL];
    bool repeat;
    
    if (where != W_HOME)
        return;

    if (pRunner->rom <= 0)
    {
        err("Your deck's ROM is completely fried. You can't make a netrun now.");
        return;
    }
    
    if (pRunner->meat <= 1)
    {
        if (!yn("You only have one meat point left. Are you sure you want to make a run?"))
        {
            return;
        }
    }

    if (pRunner->rom <= 1)
    {
        if (!yn("Your deck only has one ROM point left. Are you sure you want to make a run?"))
        {
            return;
        }
    }
    
    event("You jack in.");      
    where = W_NETSPACE;
      
    return;
}


void do_other_insert(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    bool repeat;
    
    if (where != W_HOME)
        return;

    if(pRunner->stored == NULL)
    {
        err("You have no programs to put into your cyberdeck.");
        return;
    }

    if(size_programs(pRunner) >= MAX_MU)
    {
        err("There isn't any room in your cyberdeck left.");
        return;
    }
   
    repeat = TRUE;
    while(repeat)
    {
        pProgram = input_stored_program(pRunner, "Put what program into your cyberdeck?");
        
        if (pProgram == NULL)
        {
            return;
        }
        
        if ((program_size(pProgram) + size_programs(pRunner)) > MAX_MU)
        {
            err("There isn't enough room in your cyberdeck to fit that program in.");
        }
        else
        {        
            event("You put a %s in your deck.",
                pProgram->name);
            remove_program(pRunner, pProgram);
            program_to_deck(pRunner, pProgram);
        }
    }
}


void do_other_remove(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    bool repeat;
    
    if (where != W_HOME)
        return;

    if(pRunner->programs == NULL)
    {
        err("There are no programs to remove from your cyberdeck.");
        return;
    }
        
    repeat = TRUE;
    while(repeat)
    {
        menu_init();
        menu("","Remove which?");
        display_deck_programs(pRunner);
        menu("X","Back out");
        menu_print();
        
        pProgram = input_deck_program(pRunner, "Remove what program from your cyberdeck?");
        
        if (pProgram == NULL)
        {
            return;
        }
        
        event("You remove %s from your deck.", pProgram->name);
        remove_program(pRunner, pProgram);
        program_to_stored(pRunner, pProgram);
    }
}


        
        
        








