/* Typedefs. */
typedef struct server_data SERVER_DATA;
typedef struct room_data ROOM_DATA;
typedef struct ice_master ICE_MASTER;
typedef struct ice_type_list_type ICE_TYPE_LIST;
typedef struct ice_data ICE_DATA;
typedef struct runner_data RUNNER_DATA;
typedef struct program_data PROGRAM_DATA;
typedef struct program_master PROGRAM_MASTER;
typedef struct UTIL_TYPE util_type;
typedef struct netspace_data NETSPACE_DATA;
typedef struct program_type_list_type PROGRAM_TYPE_LIST;
typedef struct netspace_type_list_type NETSPACE_TYPE_LIST;
typedef struct server_level_list_type SERVER_LEVEL_LIST;

typedef void program_fun(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram);
typedef program_fun * program_pointer_fun;
typedef void ice_fun(RUNNER_DATA * pRunner, ICE_DATA * pIce);
typedef ice_fun * ice_pointer_fun;

typedef enum
{
    R_GATE, R_WALL, R_OUT, R_NONE
}
room_type;

typedef enum
{
    S_ANTI_P, S_ANTI_D, S_ANTI_U
}
ice_type;

typedef enum
{
    FS_ANTI_P, FS_ANTI_D, FS_ANTI_U
}
ice_full_type;

typedef enum
{
    /* Anti-program */
    P_ANTI_P,
    /* Intrusion (walls) */
    P_INTR,
    /* Decryption (codegates, filelocks in future) */
    P_DECR,
    /* Protection (vs. deck & user attacks) */
    P_PROT,
    /* Stealth */
    P_STEALTH,
    /* Info (spoils) */
    P_INFO,
    /* Cred (spoils) */
    P_CRED
}
program_type;

/* For example, programs that stun ice are ANTI_P_STUN, which another table maps to ANTI_P, thus allowing them to appear under the same types of program. */
typedef enum
{
    FP_ANTI_P,    
    FP_INTR,
    FP_DECR,
    FP_PROT,
    FP_STEALTH,
    FP_INFO,
    FP_CRED
}
program_full_type;

/* Running states. */
typedef enum
{
    ST_RUNNER,
    ST_SERVER,
    ST_NEXT,
    ST_DONE
}
running_state;


/*
 * Maximums.
 */

#define MAX_ROOMS_X (9)
#define MAX_ROOMS_Y (9)
#define MAX_NAMES (26)
#define MAX_LAYOUTS (5)
#undef NET_MAX_X /*(17)*/
#undef NET_MAX_Y /*(17)*/

#define MAX_MU (10)
#define MAX_ARMOR (5)
#define MAX_RANK (5)
#define MAX_MEAT (5)
#define MAX_ROM (5)
#define MAX_LEVEL (5)
#define MAX_STR (5)

#define COST_HEAL (200)
#define COST_ROM (100)

#define INTEREST (5)
#define LOAN_INTEREST (2 * INTEREST)
#define LOAN_MAXWORTH (5) /* The percentage of their networth the runner can have loaned. */

/*
 * Direction and movement 
 */

/* Directions */
/* Do not change these! Not unless you are willing to rewrite about a third of the code! */
#define DIR_N (0)
#define DIR_E (1)
#define DIR_S (2)
#define DIR_W (3) 
#define MAX_DIR (4)

/* Used together, MAX_DIR -> x, y . */
#define dmx(x,dir) ((dir) == DIR_E ? (x) + 1 : (dir) == DIR_W ? (x) - 1 : (x))
#define dmy(y,dir) ((dir) == DIR_N ? (y) - 1 : (dir) == DIR_S ? (y) + 1 : (y))

/* Where's the runner? */
#define runner_loc() server_x, server_y
#define room_h(x,y) (room(x,y,MAX_DIR))
#define this_ice() (this_room()->ice)

/*
 * Utilities.
 */

#define has_util(util) (IS_SET(pRunner->utils,(util)))
#define add_util(util) (SET_BIT(pRunner->utils,(util)))    
#define remove_util(util) (REMOVE_BIT(pRunner->utils,(util)))    

struct UTIL_TYPE
{
    char * name;
    int bit;
    int cost;
};

#define UTIL_REPAIR    1
#define UTIL_VIRUSSCAN 2
#define UTIL_ANTIVIRUS 4
#define UTIL_NEWS      8
#define UTIL_RANKINGS  16
#define UTIL_FAKE      32
#define UTIL_AIL       64


/*
 * Servers
 */


struct server_data
{
    ROOM_DATA * rooms[MAX_ROOMS_X][MAX_ROOMS_Y];
    int level;    
    char * name;
    int start_x;
    int start_y;
};

/*
 * Rooms
 */

/* Individual rooms */

struct room_data
{
    ICE_DATA * ice;
    room_type type;
    int flags;
};

#define ROOM_OPEN  1
#define ROOM_TRIED 2
#define ROOM_CASH  4
#define ROOM_INFO  8

#define PIERCED(a) (IS_SET((a)->flags, ROOM_OPEN))
#define TRIED(a) (IS_SET((a)->flags,ROOM_TRIED) && !IS_SET((a)->flags,ROOM_OPEN))

#define this_room() (room_h(server_x, server_y))

/*
 * Ice
 */

struct ice_data
{
    char * name;
    char * action;
    char * thr;
    ice_type type;
    ice_full_type fulltype;
    int str;
    int max_str;
};

struct ice_type_list_type
{
    ice_full_type fulltype;
    ice_type type;
    char * thr;
    ice_fun * run;
};

struct ice_master
{
    char * name;
    char * action;
    ice_full_type fulltype;
    int str;
};

/* 
 * The 'runner 
 */

struct runner_data
{
    PROGRAM_DATA * programs;
    PROGRAM_DATA * stored;
    PROGRAM_DATA * last_run;
    char name[20];
    long credits;
    long deposit;
    long loan;
    int rom;
    int meat;
    int rank;
    int armor;
    int utils; /* Utils we've got */
    /* Where the 'runner is */
    int net_x;
    int net_y;
    int days; /* days played */
};

/*
 * Player programs
 */

/* Point to certain subroutines a bit more nicely. */
/* Thanks Martin. For a lot, _lot_ more than this, too. */

struct program_data
{
    char * name;
    char * action;
    char * thr;
    program_type type;
    program_full_type fulltype;
    int str; 
    int max_str;
    int cost;
    signed int duration;
    unsigned int virus;
    PROGRAM_DATA * next;
};
 
struct program_master
{
    char * name;
    char * action;
    program_full_type fulltype;
    int str;
    int cost;
};

/* Maps the fulltype to generic type, thr entry and run function. */
struct program_type_list_type
{
    char * fulltype_name;
    program_full_type fulltype;    
    program_type type;
    char * thr;
    program_fun * run;
    program_fun * run_netspace;
};


/* A bit HACKish - what if a is one of those things that should only be evaluated once? */
#define DATA(a) (((a)->type == P_INFO || (a)->type == P_CRED)) 
#define kill_program(a) \
    if ((a) != NULL) {remove_program(pRunner, (a)); free_program_data((a)); (a) = NULL;} 

/*
 * Netspace
 */

typedef enum
{
    N_NONE,
    N_LEVEL_1,
    N_LEVEL_2,
    N_LEVEL_3,
    N_LEVEL_4,
    N_LEVEL_5,
    N_NS,
    N_EW,
    N_TEE
}
netspace_type;

struct netspace_data
{
    SERVER_DATA * pServer;
    netspace_type type;
    char * name;
};

typedef enum
{
    W_START,
    W_HOME,
    W_STREET,
    W_NETSPACE,
    W_SERVER,
    W_END
}
where_state;

struct netspace_type_list_type
{
    char name;
    netspace_type type;
};

struct server_level_list_type
{
    netspace_type type;
    int level; /* Integer level, for adding to ICE str, etc. */
    char * textname; /* Name used in netspace */
    char * sname; /* Name used in title of server-running */
    char * save_name; /* Name used in save files. */
};

typedef struct txt_grid TXT_GRID;

struct txt_grid
{
    int x;
    int y;
    int filler; /* Call it a byte pad. */
    char ** grid;
};

/*
 * Misc
 */

#define is_running() (running == TRUE)
#define rom_penalty() ((MAX_ROM) - (pRunner->rom))

/* Utility macros. */
#define UMIN(a, b)              ((a) < (b) ? (a) : (b))
#define UMAX(a, b)              ((a) > (b) ? (a) : (b))
#define URANGE(a, b, c)         ((b) < (a) ? (a) : ((b) > (c) ? (c) : (b)))
#define LOWER(c)                ((c) >= 'A' && (c) <= 'Z' ? (c)+'a'-'A' : (c))
#define UPPER(c)                ((c) >= 'a' && (c) <= 'z' ? (c)+'A'-'a' : (c))
#define IS_SET(flag, bit)       ((flag) & (bit))
#define SET_BIT(var, bit)       ((var) |= (bit))
#define REMOVE_BIT(var, bit)    ((var) &= ~(bit))

#define turn_next() (next_action)
#define set_next(a) (next_action = (a))


#define IS_SERVER(a) (((a) == N_LEVEL_1)||((a) == N_LEVEL_2)||((a) == N_LEVEL_3)||((a) == N_LEVEL_4)||((a) == N_LEVEL_5))
#define IS_HIGHWAY(a) (((a) == N_NS)||((a) == N_EW)||((a) == N_TEE))

