#include "netrun.h"
/* *printf */
#include <stdio.h>

/*
 * Intro splash screen. [Hmm, could use a better splash.] :)
 */
 
void splash_screen(void)
{
    if (where != W_START)
        return;
        
    running_off();

    pr_pick(menus);
    pr_blank();

    pr("\n");
    pr("\n");
    pr("        [Hmm, could use a better title.]\n");
    pr("\n");
    pr("         ##      ## ##### ##### ####  #   # #   #  the ... game?\n");
    pr("         ###     ## #       #   #   # #   # ##  #\n");
    pr("         ####    ## #####   #   ####  #   # # # #  - . . . . . -\n");
    pr("         #####   ## #       #   #   # #   # #  ##\n");
    pr("         ## ###  ## #####   #   #   #  ###  #   #  Version 0.0.1b1\n");
    pr("         ##  ### ##\n");
    pr("         ##   #####       By Kevin Rudat <krudat@sourceforge.net>,\n");
    pr("         ##    ####\n");
    pr("         ##     ###                         and many, many others.\n");
    pr("         ##      ##\n");
    pr("\n");
    pr("   Bits (`)borrowed(') wholesale from Diku, Merc, Rom, Smaug, NiMud, \n");
    pr("circlemud, is there really enough room to fit all the author's names\n");
    pr("on the splash screen?\n");
    pr("\n");
    pr(PACKAGE ", " VERSION ".\n");

    pr_paint();

    morewait();
}

/*
 * Display player's stats.
 */
 
void display_player(RUNNER_DATA * pRunner)
{
    if (is_running())
    {
        bug("display_player: with run display\n");
        return;
    }

    menu_init();
    menu("", "Name: %s the %s. (rank %d)", 
        pRunner->name, rank_name_table[pRunner->rank], pRunner->rank);
    menu("", "Rom pts: %d/%d",          pRunner->rom, MAX_ROM);
    menu("", "Meat pts: %d/%d",         pRunner->meat, MAX_MEAT);
    menu("", "Cash: %ldcr",             pRunner->credits);
    menu("", "Deposit: %ldcr",          pRunner->deposit);
    menu("", "Loan: %ldcr",             pRunner->loan);    
    menu("", "Armor: %d",               pRunner->armor);
    menu("", "Utilities:%s.",           utilities(pRunner));
    menu("", "Days in competition: %d", pRunner->days);
    menu("", "Filename: %s",            save_file);
    menu_print();    
}

/*
 * Runner's programs, stored out of deck
 */

void display_stored_programs(RUNNER_DATA * pRunner)
{
    int i;
    PROGRAM_DATA * pProgram = NULL;
    char buf[3];
    char buf2[200];
     
    if (is_running())
    {
        bug("display_stored_programs: with run display\n");
        return;
    }
        
    i = 0;

    if(pRunner->stored == NULL)
    {
        menu("","You have no stored programs.");    
    }
    else
    {
        menu("","Stored Programs:");
        menu("No","Name          TYP STR MU");

        pProgram = pRunner->stored;

        while ( pProgram != NULL )
        {
            i++;

            sprintf(buf,"%2d",i);
             
            sprintf(buf2,"%-13s %3s %1d/%1d %2d",
                pProgram->name,
                pProgram->thr,       
                pProgram->str,
                pProgram->max_str,
                program_size(pProgram));
            
            menu(buf,buf2);
            
            pProgram = pProgram->next;
        }                    
    }
} 

/*
 * Runner's programs stored in deck
 */

void display_deck_programs(RUNNER_DATA * pRunner)
{
    int i,j;
    PROGRAM_DATA * pProgram = NULL;
    PROGRAM_DATA * pLast = NULL;
    char buf[3];
    char buf2[200];
    
    if (is_running())
    {
        bug("display_deck_programs: running");
        return;
    }
    
    menu("MU","Name          R TYP STR");

    i = 0;
    j = 0;

    pProgram = pRunner->programs; 
    
    while (pProgram != NULL )
    {
        for (j = 0; j < program_size(pProgram); j++)
        {            
            i++;
            sprintf(buf,"%2d",i);

            if (pProgram == pLast)
                sprintf(buf2,"       \"         \"  \"/\"");
            else
            {                 
                sprintf(buf2,"%-13s %1s %3s %1d/%1d",
                    pProgram->name,
                    duration_str(pProgram),
                    pProgram->thr,
                    pProgram->str,
                    pProgram->max_str);
            }
            
            menu(buf, buf2);
                        
            pLast = pProgram;        
        }
        pProgram = pProgram->next;
    }    
       
    i++;   
       
    for ( ; i <= MAX_MU; i++)
    {
        sprintf(buf,"%2d", i);
        menu(buf,"(empty)         --- -/-");
    }
} 

/* Aux function called by display_deck_programs() and display_deck_running() */
char * duration_str(PROGRAM_DATA * pProgram)
{
    if (pProgram == NULL)
    {
        return " ";        
    }

    switch (pProgram->duration)
    {
        default:
            return "?";
        case 0:
            return " ";
        case 1:
            return "1";
        case 2:
            return "2";
        case 3:
            return "3";
        case 4:
            return "4";
        case 5:
            return "5";
        case 6:
            return "6";
        case 7:
            return "7";
        case 8:
            return "8";
        case 9:
            return "9";                
        case 10:
            return "9";    
    }
}

/*
 * General function that calls all the subs needed to display the running window.
 * Called in either mode.
 */

void display_running(RUNNER_DATA * pRunner)
{
    if (!is_running())
        return;
        
    if (where != W_NETSPACE && where != W_SERVER)
        return;

    /* Wipe descs. */
    pr_pick(descs);
    pr_blank();
    pr_print();

    /* Print location ASCII graphic. */
    if (where == W_SERVER)
        display_room(runner_loc());    
    else if (where == W_NETSPACE)
        display_net_space(pRunner);        

    /* Show current space contents. */
    if (where == W_SERVER)
        show_room(pRunner);    
    else if (where == W_NETSPACE)
        show_netspace(pRunner);

    /* Display the menu. */
    display_running_menu(pRunner);
    /* Show runner programs. */
    display_programs_running(pRunner);
    /* Show runner stats. */
    show_stats(pRunner);    

    /* The above functions call wnoutrefresh - now we call doupdate. Efficiency. */
    doupdate();
}

/*
 * Graphically (ASCII-art!) displays server rooms.
 */

void display_room(int x, int y)
{
    ROOM_DATA * pRoom;
    int x2, y2;
    
    if (!is_running())
        return;
    
    if (where != W_SERVER)
        return;

    pr_pick(display);                
    pr_blank();

    /*
     * Frame.
     */
    
    prc(ACS_ULCORNER); rch(27, ACS_HLINE);
    
    {
        int i;
        
        for (i = 0; i < 12; i++)
        {
            prm(i + 1, 0);
            prc(ACS_VLINE);
        }
    }

    prm(13, 0);
    prc(ACS_LLCORNER);
    rch(27, ACS_HLINE);

    /* 
     * Content
     */


    /* The header. */     
    prm(1, 2); pr("X: %1d Y: %1d      %5s",
        x + 1, y + 1,
        server_level_name(server->level)->sname);
    prm(2, 2); pr("%s\n", server->name);    

    prm(4, 0);

    /* NW */

    pRoom = room_h(dmx(x, DIR_W), dmy(y,DIR_N));
    
    prm(4, 9);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr(".#");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr(".-");
                else
                    pr(".|");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("|+");
                else
                    pr("-+");
                break;
            case R_NONE:
                pr(".+");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }

    
    /* N */
    
    pRoom = room_h(dmx(x, DIR_N), dmy(y,DIR_N));

    prm(4, 11);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else    
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.....#");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.....-");
                else
                    pr("|.....|");
                break;
            case R_NONE:
            case R_OUT:
                pr(".......");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("##...##");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|...|+");
                else
                    pr("+--|--+");
                break;
            case R_NONE:
                pr("+.....+");
                break;
            case R_OUT:
                pr(".......");
                break;
        }
    }

    /* NE */
    
    pRoom = room_h(dmx(x, DIR_E), dmy(y,DIR_N));
    prm(4, 18);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.");
                else
                    pr("|.");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|");
                else
                    pr("+-");
                break;
            case R_NONE:
                pr("+.");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }
    
    /* W */
     
    pRoom = room_h(dmx(x, DIR_W), dmy(y, DIR_W));

    prm(6, 9);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("|+");
                else
                    pr("-+");
                break;
            case R_NONE:
                pr(".+");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr(".#");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr(".-");
                else
                    pr(".|");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("..");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("..");
                else
                    pr(".-");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr(".#");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr(".-");
                else
                    pr(".|");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            default:
                pr("]]");
                break;
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("|+");
                else
                    pr("-+");
                break;
            case R_NONE:
                pr(".+");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }

    
    /* MAX_DIR */

    pRoom = room_h(dmx(x, MAX_DIR), dmy(y, MAX_DIR));

    prm(6, 11);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("##...##");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|...|+");
                else
                    pr("+--|--+");
                break;
            case R_NONE:
                pr("+.....+");
                break;
            case R_OUT:
                pr(".......");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.....#");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.....-");
                else
                    pr("|.....|");
                break;
            case R_NONE:
            case R_OUT:
                pr(".......");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("   !   ");
    }
    else
    {
        switch(pRoom->type)
        {
            default:
                pr("|]]*[[|");
                break;
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("...*...");
                else
                    pr("###!###");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("...*...");
                else
                    pr("-..*..-");
                break;
            case R_NONE:
            case R_OUT:
                pr("...*...");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            default:
                pr("|]]#[[|");
                break;
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.....#");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.....-");
                else
                    pr("|.....|");
                break;
            case R_NONE:
            case R_OUT:
                pr(".......");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("##...##");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|...|+");
                else
                    pr("+--|--+");
                break;
            case R_NONE:
                pr("+.....+");
                break;
            case R_OUT:
                pr(".......");
                break;
        }
    }

    
    /* E */
    
    pRoom = room_h(dmx(x, DIR_E), dmy(y, DIR_E));

    prm(6, 18);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(room_h(dmx(x, DIR_E), dmy(y, DIR_E))))
                    pr("+|");
                else
                    pr("+-");
                break;
            case R_NONE:
                pr("+.");
                break;
            case R_OUT:
                pr("..");            
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.");
                else
                    pr("|.");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("..");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("..");
                else
                    pr("-.");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            default:
                pr("]]");
                break;
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.");
                else
                    pr("|.");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|");
                else
                    pr("+-");
                break;
            case R_NONE:
                pr("+.");
                break;
            case R_OUT:
                pr("..");            
                break;
        }
    }
    
    /* SW */
     
    pRoom = room_h(dmx(x, DIR_W), dmy(y,DIR_S));
    prm(11, 9);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("|+");
                else
                    pr("-+");
                break;
            case R_NONE:
                pr(".+");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(room_h(dmx(x, DIR_W), dmy(y,DIR_S))))
                    pr(".#");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(room_h(dmx(x, DIR_W), dmy(y,DIR_S))))
                    pr(".-");
                else
                    pr(".|");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }

    
    /* S */
    
    pRoom = room_h(dmx(x, DIR_S), dmy(y,DIR_S));

    prm(11, 11);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("##...##");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|...|+");
                else
                    pr("+--|--+");
                break;
            case R_NONE:
                pr("+.....+");
                break;
            case R_OUT:
                pr(".......");
                break;
        }
    }
    prd(1, -7);
    if (pRoom == NULL)
    {
        pr("       ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.....#");
                else
                    pr("#######");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.....-");
                else
                    pr("|.....|");
                break;
            case R_NONE:
            case R_OUT:
                pr(".......");
                break;
        }
    }

    /* SE */

    pRoom = room_h(dmx(x, DIR_E), dmy(y,DIR_S));
     
    prm(11, 18);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("+|");
                else
                    pr("+-");
                break;
            case R_NONE:
                pr("+.");
                break;
            case R_OUT:
                pr("..");
                break;
        }
    }
    prd(1, -2);
    if (pRoom == NULL)
    {
        pr("  ");
    }
    else
    {
        switch(pRoom->type)
        {
            case R_WALL:
                if (PIERCED(pRoom))
                    pr("#.");
                else
                    pr("##");
                break;
            case R_GATE:
                if (PIERCED(pRoom))
                    pr("-.");
                else
                    pr("|.");
                break;
            case R_NONE:
            case R_OUT:
                pr("..");
                break;
        }
    }
    

    pr_print();
}

/*
 * Displays 5x5 grid of netspaces for server-server netspace mode.
 */

void display_net_space(RUNNER_DATA * pRunner)
{
    /* HACK Hardcoded directions. */
    
    if (where != W_NETSPACE)
        return;
    
    if (!is_running())
        return;

    /*
     * Frame
     */
 
    pr_pick(display);
    pr_blank();

    prc(ACS_ULCORNER); rch(27, ACS_HLINE);
    
    {
        int i;
        
        for (i = 0; i < 12; i++) /* HACK Hardcoded value */
        {
            prm(i + 1, 0); prc(ACS_VLINE);
        }
    }

    prm(13, 0);  prc(ACS_LLCORNER); rch(27, ACS_HLINE);

    /*
     * Content
     */    
    
    prm(1, 2);
    pr("X: %2d Y: %2d",
        (pRunner->net_x) - (net_size_x / 2), 
        (pRunner->net_y) - (net_size_y / 2));

    /* Loop out map. */
    {
        int x;
        int y;
        
        for (y = 0; y < 5; y++) 
        {
            for(x = 0; x < 5; x++) 
            {
                prm(y * 2 + 3, x * 4 + 5);
                disp_netspace(pRunner, pRunner->net_x + (x - 2), pRunner->net_y + (y - 2));
            }
        }
    }

    pr_print();
}

/* Non-checking auxillary to display_net_space(), that prints an individual netspace. */
void disp_netspace(RUNNER_DATA * pRunner, int x, int y)
{
    NETSPACE_DATA * pNetspace;
    
    pNetspace = net_space(x, y, MAX_DIR);
    
    if (pNetspace == NULL)
    {
        pr("   ");
        prd(1, - 3);
        pr("   ");
    }    
    else if (IS_SERVER(pNetspace->type))
    {
        if (pNetspace->pServer != NULL)
            pr("]");
        else    
            pr("#");    

        pr("%c", UPPER(pNetspace->name[0]));

        if (pNetspace->pServer != NULL)
            pr("[");
        else    
            pr("#");    

        prd(1, - 3);

        pr("#");

        switch(pNetspace->type)
        {
            case N_LEVEL_1:
                pr("#");
                break;
            case N_LEVEL_2:
                pr("1");
                break;
            case N_LEVEL_3:
                pr("2");
                break;
            case N_LEVEL_4:
                pr("3");
                break;
            case N_LEVEL_5:
                pr("x");
                break;
        }                    

        pr("#");

    }
    else if (pNetspace->type == N_NONE)
    {
        pr("...");
        prd(1, - 3);
        pr("...");    
    }
    else if (pNetspace->type == N_NS)
    {
        pr(".");
        prc(ACS_VLINE);
        pr(".");
        prd(1, - 3);
        pr(".");
        prc(ACS_VLINE);
        pr(".");
    }
    else if (pNetspace->type == N_EW)
    {
        /* I don't like doing them like this, but I can't think of a better alternative. */
        bool west = FALSE, east = FALSE;
        
        if (net_space(x, y, DIR_W) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_W)->type))
                west = TRUE;

        if (net_space(x, y, DIR_E) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_E)->type))
                east = TRUE;
    
        pr("...");
        prd(1, - 3);    

        if (west)
        {
            prd(0, -1);
            prc(ACS_HLINE);
        }
            
        rch(3, ACS_HLINE);    

        if (east)
        {
            prc(ACS_HLINE);
            prd(0, -1);
        }
    }
    else if (pNetspace->type == N_TEE) /* A complex hack. HACK UNFIN Just a generic tee right now... to do this properly we must pass on the x and y. Argh. */
    {        
        bool west = FALSE;
        bool east = FALSE;
        bool north = FALSE;
        bool south = FALSE;
        
        if (net_space(x, y, DIR_W) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_W)->type))
                west = TRUE;

        if (net_space(x, y, DIR_E) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_E)->type))
                east = TRUE;

        if (net_space(x, y, DIR_N) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_N)->type))
                north = TRUE;

        if (net_space(x, y, DIR_S) != NULL)
            if (IS_HIGHWAY(net_space(x, y, DIR_S)->type))
                south = TRUE;

        pr (".");

        if (north)
            prc(ACS_VLINE);
        else
            pr(".");
        
        pr(".");

        prd(1, - 3);
        
        if (west)
            prc(ACS_HLINE);
        else
            pr(".");

        /* HACK Name me a better way of doing this - I'm all ears! */
        if (!north && !south && !east && !west)
            pr(".");
        else if (!north && !south && !east && west)
            pr(".");
        else if (!north && !south && east && !west)
            pr(".");
        else if (!north && !south && east && west)
            prc(ACS_HLINE);
        else if (!north && south && !east && !west)
            pr(".");
        else if (!north && south && !east && west)
            prc(ACS_URCORNER);
        else if (!north && south && east && !west)
            prc(ACS_ULCORNER);
        else if (!north && south && east && west)
            prc(ACS_TTEE);
        else if (north && !south && !east && !west)
            pr(".");
        else if (north && !south && !east && west)
            prc(ACS_LRCORNER);
        else if (north && !south && east && !west)
            prc(ACS_LLCORNER);
        else if (north && !south && east && west)
            prc(ACS_BTEE);
        else if (north && south && !east && !west)
            prc(ACS_VLINE);
        else if (north && south && !east && west)
            prc(ACS_RTEE);
        else if (north && south && east && !west)
            prc(ACS_TTEE);            
        else if (north && south && east && west)
            prc(ACS_PLUS);

        if (east)
            prc(ACS_HLINE);
        else
            pr(".");
        
    }
    
    prd(0, -2);
    
    /* Now for some more specific hacks. */    
    if (net_space(pRunner->net_x, pRunner->net_y, MAX_DIR) == pNetspace)
    {
        if (pNetspace == NULL)
            pr("!");
        else    
            pr("*");
    }
    else if (net_space(start_net_x, start_net_y, MAX_DIR) == pNetspace)
    {
        pr("^"); /* We're not kidding. */
    }
}

/* 
 * Gives text description of a given server room.
 */

void show_room(RUNNER_DATA * pRunner)
{    
    int i;

    desc_clear();        

    for (i = 0; i < MAX_DIR; i++)
    {
        if (room(runner_loc(),i) != NULL)
        {    
            if (TRIED(room(runner_loc(),i)) && !PIERCED(room(runner_loc(),i)))
            {                
                desc("You have tried and failed to breach the %s ",
                    dir_name_table[i]);

                switch(room(runner_loc(),i)->type)
                {
                    case R_WALL:
                        desc("section of data wall");   
                        break;   
                    case R_GATE:
                        desc("codegate");   
                        break;   
                    case R_OUT:
                    case R_NONE:
                        desc("bug");
                        break;    
                }


                desc(".\n");
            }    
        }
    }
    
    if (this_room()->ice != NULL)
    {
        desc("A %s watches %swarily.\n",
            ice_long(this_room()->ice),
            stealth_bonus(pRunner) ? "" : "you ");
    }
    
    if (IS_SET(this_room()->flags,ROOM_CASH) || IS_SET(this_room()->flags,ROOM_INFO))
    {
        desc("An interesting file catches your attention.\n");
    }
    
    wnoutrefresh(descs);
}

/* 
 * Netspace equivalent of show_room - gives the names, ranks and one other detail of current and adjacent servers.
 */
void show_netspace(RUNNER_DATA * pRunner)
{
    int i;

    show_netspace_aux(net_space(pRunner->net_x, pRunner->net_y, MAX_DIR),
        "here");
    
    for (i = 0; i < MAX_DIR; i++)
    {
        show_netspace_aux(net_space(pRunner->net_x, pRunner->net_y, i), 
            dir_name_table[i]);        
    }
}

/*
 * Auxillary function to show_netspace(). Prints direction, name, and rank of server, and if the player has run against it ~today~.
 */
void show_netspace_aux(NETSPACE_DATA * pNetspace, const char * where)
{
    if (pNetspace == NULL)
        return;    
    
    if (!IS_SERVER(pNetspace->type))
        return;
    
    desc(where);
    desc(": ");
 
    desc(pNetspace->name);
    
    desc(", ");
    
    desc(server_level_name(netspace_level(pNetspace))->textname);
    
    desc(".");

    if (pNetspace->pServer != NULL)
    { 
        desc(" You've run against it today.");
    }
    
    desc("\n");            
}

SERVER_LEVEL_LIST * server_level_name(int level)
{
    int i;
    
    i = 0;
    
    while(server_level_list[i].type != N_NONE)
    {
        if (server_level_list[i].level == level)
            return &server_level_list[i];
        
        i++;
    }
    
    return NULL;
}

/*
 * Prints the menu that sits between the display and the programs.
 */

void display_running_menu(RUNNER_DATA * pRunner)
{
    /*
     * Frame
     */

    pr_pick(menus);     
    pr_blank();
    prm(0, 0); prc(ACS_TTEE); rch(14, ACS_HLINE); prc(ACS_TTEE);
    prm(1, 0); prc(ACS_VLINE); prm(1, 15); prc(ACS_VLINE);
    prm(2, 0); prc(ACS_VLINE); prm(2, 15); prc(ACS_VLINE);
    prm(3, 0); prc(ACS_VLINE); prm(3, 15); prc(ACS_VLINE);
    prm(4, 0); prc(ACS_VLINE); prm(4, 15); prc(ACS_VLINE);
    prm(5, 0); prc(ACS_VLINE); prm(5, 15); prc(ACS_VLINE);
    prm(6, 0); prc(ACS_VLINE); prm(6, 15); prc(ACS_VLINE);
    prm(7, 0); prc(ACS_VLINE); prm(7, 15); prc(ACS_VLINE);
    prm(8, 0); prc(ACS_VLINE); prm(8, 15); prc(ACS_VLINE);
    prm(9, 0); prc(ACS_LTEE); rch(14, ACS_HLINE); prc(ACS_RTEE);

    /* 
     * Content
     */

    prm(1, 2); pr("[R] Run");
    prm(2, 2); pr("[C] Copy");
    prm(3, 2); pr("[D] Delete");

    prm(4, 2);
    pr("[J] Jack ");

    if (!IS_SERVER(net_space(pRunner->net_x, pRunner->net_y, MAX_DIR)->type) || where == W_SERVER)
        pr("Out");
    else
        pr("In");

    prm(5, 2); pr("[N] North");
    prm(6, 2); pr("[S] South");
    prm(7, 2); pr("[E] East");
    prm(8, 2); pr("[W] West");

    pr_print();    
}

/*
 * Prints list of runner's programs-in-deck, for/in running mode.
 */

void display_programs_running(RUNNER_DATA * pRunner)
{
    int i,j;
    PROGRAM_DATA * pProgram = NULL;
    PROGRAM_DATA * pLast = NULL;
    
    if (!is_running())
    {
        bug("display_programs_running: not running");
        return;
    }
    
    if (where != W_NETSPACE && where != W_SERVER)
        return;

    pr_pick(programs);
    pr_blank();

    /*
     * Frame
     */

    rch(31, ACS_HLINE); prc(ACS_URCORNER);    

    for(i = 0; i < 12; i++)
    {
        prm(i + 1, 31); prc(ACS_VLINE);
    }

    prm(13, 0); rch(31, ACS_HLINE); prc(ACS_LRCORNER);        

    /*
     * Content
     */

    prm(1, 1); pr("-*- Programs:");
    prm(2, 1); pr("[MU] Name             TYP STR");

    i = 0;
    j = 0;

    pProgram = pRunner->programs; 
    
    while (pProgram != NULL )
    {
        for (j = 0; j < program_size(pProgram); j++)
        {            
            prm(i + 3, 1);
            i++;
            if (pProgram == pLast)
                pr("[%2d]                   \"   \"", i);
            else
            {                 
                pr("[%2d] %-15s%1s %3s %1d/%1d",
                    i,
                    pProgram->name,
                    duration_str(pProgram),
                    pProgram->thr,
                    pProgram->str,
                    pProgram->max_str);
            }
            
            pLast = pProgram;        
        }
        pProgram = pProgram->next;
    }    
    
    i++;
       
    for ( ; i <= MAX_MU; i++)
    {
        prm(i + 2, 1);
        pr("[%2d] (empty)          --- -/-", i);
    }

    pr_print();
} 


/*
 * Display the player's rom, health and armor in running mode.
 */
 
void show_stats(RUNNER_DATA * pRunner)
{
    if (is_running() && !(where != W_NETSPACE && where != W_SERVER))
    {        
        pr_pick(player_stats);
        pr_blank();
        
        /*
         * Frame
         */
        
        prm(0, 0); prc(ACS_VLINE); prm(0, 15); prc(ACS_VLINE);
        prm(1, 0); prc(ACS_VLINE); prm(1, 15); prc(ACS_VLINE);
        prm(2, 0); prc(ACS_VLINE); prm(2, 15); prc(ACS_VLINE);
        prm(3, 0); prc(ACS_BTEE); rch(14, ACS_HLINE); prc(ACS_BTEE);
        
        /* 
         * Content 
         */
        
        prm(0, 2); pr("ROM:     %1d/%1d", pRunner->rom, MAX_ROM);
        prm(1, 2); pr("Meat:    %1d/%1d", pRunner->meat, MAX_MEAT);
        prm(2, 2); pr("Armor:     %1d", pRunner->armor);

        pr_print();
    }
}

