#include "netrun.h"
#include <stdio.h>
/* for *(fmt,...) */
#include <stdarg.h>
/* atoi */
#include <stdlib.h>

/* 
 * Gets a string from the console (blahblahblah\n).
 *
 * Returns a pointer to a static buffer, with the \n stripped.
 */
char * input_long(char * fmt, ...)
{
    char buf[200];
    static char buf2[200];
    va_list args;
    int i;
    char byte = 'x';

    /* Collect arguments. I hope to see no use of sprintf in the main code. */        
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    werase(inputs);
    wmove(inputs, 0, 0);
    waddstr(inputs, buf);
    waddstr(inputs, " > ");
    wrefresh(inputs);
    
    /* HACK somehow this stinks. Heavily. */
    buf2[0] = '\0';
    i = 0;

    byte = wgetch(inputs);
    while (byte != '\n')
    {
        /* We got a keystroke, blank the errors. */
        err_blank();
        
        /* ^h or delete */
        /* On backspace or delete, back up the pointer (if applicable) and erase the character from both screen and buffer. */
        if ((byte == 8) || (byte == 127))
        {
            if ( i > 0 )
            {
                i--;
                {
                    int x,y;

                    getyx(inputs,y,x);

                    if (x > 0)
                    {
                        waddch(inputs, ' ');
                        wmove(inputs, y, x - 1);
                        waddch(inputs, ' ');
                        wmove(inputs, y, x - 1);
                    }
                    wrefresh(inputs);
                }
            }
            buf2[i] = 0;
        }        
        /* HACK */
        /* ^L -> refresh screen. */
        else if (byte == 12)
        {
            wrefresh(curscr);
        }
        else
        {
            buf2[i] = byte;
            waddch(inputs,byte);
            wrefresh(inputs);
            i++;
            buf2[i] = '\0';
        }
        byte = wgetch(inputs);
    }   

    /* Kill ending '\n's. */
    for (i = 200; i > 0; i--)
    {
        if(buf2[i] == '\n')
        {
            buf2[i] = '\0';
        }   
    }
   
    return buf2;
}



/*
 * Gets a single keystroke.  
 *
 * Returns a pointer to a static buffer containing _one_ character, being the
 * keystroke, capitalised. This is important when
 * comparing input() to another string.
 *
 * Display a question on the second-last line of the screen, and loop
 * until we get a keystroke other than ^L. If it's ^L, refresh the screen
 * and continue to loop, otherwise return a pointer to a static buffer
 * containing the keystroke.
 *
 * HACK uses a non-portable method of detecting ^L. char n; n = wgetch(inputs); if (n == 12) {...
 */
char * input(char * fmt, ...)
{
    static char buf2[2];
    char buf[200];
    va_list args;
    char choice;
    bool repeat;

    /* Collect arguments. I hope to see no use of sprintf in the main code. */
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    /* Clear the input line. Display the question, and a prompt. */
    werase(inputs);
    wmove(inputs, 0, 0);
    waddstr(inputs, buf);
    waddstr(inputs, " > ");

    /* Now, loop! */

    repeat = TRUE;
    while (repeat)
    {
        choice = wgetch(inputs);

        /* HACK */       
        /* If wgetch is ^L */ 
        if (choice == 12)
        {
            /* Repaint the screen. I _think_ this is the proper way of doing it - it is pointed out with no warnings in the libncurses4-dev man pages. */
            wrefresh(curscr);
        }
        /* We got a character to pass back, get the heck out. */
        else
            repeat = FALSE;
    }
    
    /* We got a keystroke - blank the error messages. */
    err_blank();    

    /* Shove the keystroke into the return static buffer. Made uppercase to make sure one doesn't end up having to check against both upper and lowercase keystrokes in menus (or, more likely, have to put the UPPER in oneself.) */       
    buf2[0] = UPPER(choice);    

    /* Just to be on the safe side. It kept on returning one character too many, somehow. */
    buf2[1] = '\0';
    
    /* All done, pass the character. */
    return buf2;            
}

/*********************/
/* High-level inputs */
/*********************/

/* Yes, or no? */
bool yn(char * fmt, ...)
{
    char buf[200];
    va_list args;

    /* Collect arguments. I hope to see no use of sprintf in the main code. */    
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    /* HACK Maybe I should use a repeat-loop and else if chain, to make it consistent? */
    /* Any other response than 'yes' is 'no'. */
    return (!strcmp("Y", input("%s [yn]", buf)));
} 

/*
 * Get a number. 
 * Returns a number between llimit and ulimit (inclusive), or a number less
 * than llimit if the player backs out.
 */
int input_num(int llimit, int ulimit, char * fmt, ...)
{
    char buf[200];
    va_list args;
    bool repeat;
    int c;
    char choice[200];

    /* Collect question arguments. */
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    /* Bad limits? */
    if (ulimit < llimit)
    {
        bug("input_num: bad llimit and/or ulimit, %d %d", llimit, ulimit);
        return (llimit - 1);
    }

    repeat = TRUE;
    while(repeat)
    {
        /* If entry is only one digit, use input(), otherwise input_long(). */        
        /* HACK doubled hardcoded code, ah, yeah. */
        if ((llimit > -10) && (ulimit < 10))
            strcpy(choice, input("%s [%d-%d,x]",buf,llimit,ulimit));
        else
            strcpy(choice, input_long("%s [%d-%d,x]",buf,llimit,ulimit));
        
        if (UPPER(choice[0]) == 'X')
            return (llimit - 1);
        
        c = atoi(choice);
        
        if ((c >= llimit) && (c <= ulimit))
            repeat = FALSE;
    }
    return c;
}


/*********************/
/* Game-level inputs */
/*********************/

/*
 * Selecting programs
 */

PROGRAM_DATA * input_stored_program(RUNNER_DATA * pRunner, char * question)
{
    PROGRAM_DATA * pProgram = NULL;
    int i,c;
    bool repeat;
            
    if (is_running())
    {
        bug("input_stored_program: with run display\n");
        return NULL;
    }
    
    if (pRunner->stored == NULL)
        return NULL;
    
    repeat = TRUE;
    
    while(repeat)
    {    
        menu_init();
        menu("",question);
        display_stored_programs(pRunner);
        menu("X","Back out");
        menu_print();

        c = input_num(1, num_stored_programs(pRunner), question);        

        if (c < 1)
        {
            repeat = FALSE;
        }

        i = 0;

        pProgram = pRunner->stored;

        while ( pProgram != NULL )
        {
            i++;
            if (i == c)
                return pProgram;
            else  /* paranoid */
                pProgram = pProgram->next;
        }
        

    }    

    return NULL;
}


PROGRAM_DATA * input_deck_program(RUNNER_DATA * pRunner, char * question)
{
    PROGRAM_DATA * pProgram = NULL;
    int i,c,j;
    bool repeat;

    if (pRunner->programs == NULL)
        return NULL;
                    
    repeat = TRUE;
    
    while(repeat)
    {            
        c = input_num(1, size_programs(pRunner), question);

        if ( c < 1)
        {
            repeat = FALSE;
        }
        
        i = 0;
        pProgram = pRunner->programs;

        while ( pProgram != NULL )
        {
            for(j = 0; j < program_size(pProgram); j++)
            {
                i++;
                if (i == c)
                    return pProgram;
            }
            
            pProgram = pProgram->next;
        }        
        
    }    

    return NULL;
}
 
/*
 * Get a direction. Returns MAX_DIR on input of 'X'.
 */

int input_direction(char * fmt, ...)
{
    char buf[200];
    va_list args;
    int direction;
    char * choice;
    bool repeat;

    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    repeat = TRUE;
    while(repeat)
    {        
        choice = input("%s which way? [nsewx]",buf);

        if (!strcmp(choice,"N") || !strcmp(choice,"8"))
        {
            direction = DIR_N;
            repeat = FALSE; 
        }
        else if (!strcmp(choice,"E") || !strcmp(choice,"6"))
        {
            direction = DIR_E;
            repeat = FALSE;
        }
        else if (!strcmp(choice,"S") || !strcmp(choice,"2"))
        {
            direction = DIR_S;
            repeat = FALSE;
        }
        else if (!strcmp(choice,"W") || !strcmp(choice,"4"))
        {
            direction = DIR_W;
            repeat = FALSE;
        }
        else if (!strcmp(choice,"X"))
        {
            direction = MAX_DIR;
            repeat = FALSE;
        }
        else
        {
            err("%s which way?",buf);
        }
    }
    
    return direction;
}

 
