#include "netrun.h"
/* malloc, free */
#include <stdlib.h>

/*
 * Allocate and free non-permenant memory.
 */

/*
 * Reasonably low-level wrapper stuff.
 *
 * TODO Make it more robust, or something like that. I don't know how, however.
 */

void * alloc_mem(size_t size)
{
    void * allocated;
    
    allocated = malloc(size);
    
    /* Out of memory! */
    if (allocated == NULL)
    {
        /*bug();*/
        /*quit_cleanly(0);*/
    }
    
    return allocated;
}

void * calloc_mem(size_t nmemb, size_t size)
{
    void * allocated;
    
    allocated = calloc(nmemb, size);
    
    /* Out of memory! */
    if (allocated == NULL)
    {
        /*bug();*/
        /*quit_cleanly(0);*/
    }
    
    return allocated;
}

void free_mem(void * ptr)
{
    if (ptr == NULL)
    {
        /*bug();*/
        return;
    }

    free(ptr);    
}


PROGRAM_DATA * new_program_data(void)
{
    PROGRAM_DATA * pProgram;
    
    pProgram = alloc_mem(sizeof(*pProgram));
    
    if (pProgram == NULL)
    {
        bug("new_program_data: out of memory!!");
    }
    
    pProgram->name = NULL;
    pProgram->action = NULL;
    pProgram->thr = NULL;
    pProgram->type = P_CRED;
    pProgram->fulltype = P_CRED;    
    pProgram->str = 0;
    pProgram->max_str = 0;
    pProgram->cost = 0;
    pProgram->duration = 0;
    pProgram->virus = 0;
    pProgram->next = NULL;
    
    return pProgram;
}

void free_program_data(PROGRAM_DATA *pProgram)
{
    if (pProgram == NULL)
    {
        bug("free_program_data: null arg");
        return;
    }

    pProgram->name = NULL;
    pProgram->action = NULL;
    pProgram->thr = NULL;
    pProgram->type = P_CRED;
    pProgram->fulltype = P_CRED;    
    pProgram->str = 0;
    pProgram->max_str = 0;
    pProgram->cost = 0;
    pProgram->duration = 0;
    pProgram->virus = 0;
    /* A bit of recursion. People had better remeber to use kill_program, else a lot of players are going to mysteriously lose most of their programs... */
    if (pProgram->next != NULL)
        free_program_data(pProgram->next);
    pProgram->next = NULL;

    free_mem(pProgram);
    
    return;
}

ICE_DATA * new_ice_data(void)
{
    ICE_DATA * pIce;
    
    pIce = alloc_mem(sizeof(*pIce));
    
    if (pIce == NULL)
    {
        bug("new_ice_data: out of memory!!");
    }

    pIce->name = NULL;
    pIce->action = NULL;
    pIce->thr = NULL;
    pIce->type = 0;
    pIce->fulltype = 0;
    pIce->str = 0;
    pIce->max_str = 0;    

    return pIce;
}

void free_ice_data(ICE_DATA *pIce)
{
    if (pIce == NULL)
    {
        bug("free_ice_data: null arg");
        return;
    }
    
    pIce->type = 0;
    pIce->max_str = 0;
    pIce->str = 0;
    
    free_mem(pIce);
    
    return;
}

ROOM_DATA * new_room_data(void)
{
    ROOM_DATA * pRoom;
    
    pRoom = alloc_mem(sizeof(*pRoom));
    
    if (pRoom == NULL)
    {
        bug("new_room_data: out of memory!!");        
    }
    
    pRoom->ice = NULL;
    pRoom->type = R_OUT;
    pRoom->flags = 0;
    
    return pRoom;
}

void free_room_data(ROOM_DATA * pRoom)
{
    if (pRoom == NULL)
    {
        bug("free_room_data: null arg");
        return;
    }
    
    if (pRoom->ice != NULL)
        free_ice_data(pRoom->ice);
    pRoom->ice = NULL;
    pRoom->type = R_OUT;
    pRoom->flags = 0;

    free_mem(pRoom);
    
    return;
}
    
SERVER_DATA * new_server_data(void)
{
    SERVER_DATA * pServer;
    int x, y;
    
    pServer = alloc_mem(sizeof(*pServer));
    
    if (pServer == NULL)
    {
        bug("new_server_data: out of memory!!");
    }    

    for (x = 0; x < MAX_ROOMS_X; x++)
    {
        for( y = 0; y < MAX_ROOMS_Y; y++)
        {
            pServer->rooms[x][y] = NULL;
        }
    }

    pServer->level = 0;
    pServer->name = NULL;
    pServer->start_x = 0;
    pServer->start_y = 0;
    
    return pServer;
}

void free_server_data(SERVER_DATA * pServer)
{
    int x, y;

    if (pServer == NULL)
    {
        bug("free_server_data: null arg");
        return;
    }

    for (x = 0; x < MAX_ROOMS_X; x++)
    {
        for( y = 0; y < MAX_ROOMS_Y; y++)
        {
            if (pServer->rooms[x][y] != NULL)
                free_room_data(pServer->rooms[x][y]);
            pServer->rooms[x][y] = NULL;
        }
    }

    pServer->level = 0;
    pServer->name = NULL;
    pServer->start_x = 0;
    pServer->start_y = 0;
    
    free_mem(pServer);
    
    return;
}

RUNNER_DATA * new_runner_data(void)
{
    RUNNER_DATA * pRunner;
    
    pRunner = alloc_mem(sizeof(*pRunner));
    
    if (pRunner == NULL)
    {
        bug("new_runner_data: out of memory!!");
    }
    
    pRunner->programs = NULL;
    pRunner->stored = NULL;
    pRunner->last_run = NULL;
    pRunner->name[0] = '\0';
    pRunner->credits = 0;
    pRunner->deposit = 0;
    pRunner->loan = 0;
    pRunner->rom = 0;
    pRunner->meat = 0;
    pRunner->rank = 0;
    pRunner->armor = 0;
    pRunner->utils = 0;
    pRunner->net_x = 0;
    pRunner->net_y = 0;
    pRunner->days = 0;
    
    return pRunner;
}

/* HACK UNFIN this will not properly unallocate carried programs.
   free_program_data does not (and _must_ not) recurse, and for some reason 
   recursing through linked lists doesn't seem to work properly.*/
   
void free_runner_data(RUNNER_DATA * pRunner)
{
    if (pRunner == NULL)
    {
        bug("free_runner_data: null arg");
        return;
    }

    if (pRunner->programs != NULL)
        free_program_data(pRunner->programs);    
    pRunner->programs = NULL;

    if (pRunner->stored != NULL)
        free_program_data(pRunner->stored);    
    pRunner->stored = NULL;

    pRunner->last_run = NULL;
    pRunner->name[0] = '\0';
    pRunner->credits = 0;
    pRunner->deposit = 0;
    pRunner->loan = 0;
    pRunner->rom = 0;
    pRunner->meat = 0;
    pRunner->rank = 0;
    pRunner->armor = 0;
    pRunner->utils = 0;
    pRunner->net_x = 0;
    pRunner->net_y = 0;
    pRunner->days = 0;
    
    free_mem(pRunner);
    
    return;
}
    
TXT_GRID * new_txt_grid(void)    
{
    TXT_GRID * pGrid;
    
    pGrid = alloc_mem(sizeof(*pGrid));
    
    if (pGrid == NULL)
    {
        bug("new_txt_grid: out of memory!!");
    }
    
    pGrid->x = 0;
    pGrid->y = 0;
    pGrid->grid = NULL;
    
    return pGrid;
}
 
void free_txt_grid(TXT_GRID * pGrid)
{
    if (pGrid == NULL)
    {
        bug("free_txt_grid: null arg");
        return;
    }
    
    if (pGrid->grid != NULL)
        free_mem(pGrid->grid);
    pGrid->grid = NULL;
    
    pGrid->x = 0;
    pGrid->y = 0;

    free_mem(pGrid);    
    
    return;
}






