#include "netrun.h"

/*
 * This (a) is used for both sneaking and ordinary movement, and (b) is used
 * for all directions, hence the 'direction' argument.
 */

void do_running_move(RUNNER_DATA * pRunner, int direction)
{
    /* HACK */

    if (room(runner_loc(), direction) == NULL)
    {
        desc("You leave the vicinity of the server, returning to Netspace.\n");
        morewait(); /* So players see it. */
        where = W_NETSPACE;
        server = NULL;
        return;
    }
   

   if(check_move(runner_loc(), direction))
   {
       if (ice_check())
       {
           if (stealth_bonus(pRunner))
           {       
               if (stealth_check(pRunner))
               {
                   desc("You successfully sneak past the %s to the %s.\n",
                       this_ice()->name,
                       dir_name_table[direction]);
               }
               else
               {
                   desc("You fail to sneak past the %s to the %s. Oops.\n",
                       this_ice()->name,
                       dir_name_table[direction]);
                   set_next(ST_SERVER);
                   return;
               }
           }
           else
           {
               err("You can't just walk away from the %s - it won't let you.\n",
                   this_ice()->name);
               return;              
           }
       }

       server_x = dmx(server_x, direction);
       server_y = dmy(server_y, direction);
       set_next(ST_NEXT);
   }   
}



/*
 * Checks whether or not the 'runner can move in a given direction from a given space.
 * Uses err to indicate why a given move would fail. Failed moves do _not_ cost an action.
 * Note that this does not check for the presence of a server program - that's up to the caller.
 */

bool check_move(int x, int y, int dir)
{
    ROOM_DATA * pRoom;

    /* Are we anywhere? */
    pRoom = room_h(x,y);
    
    if (pRoom == NULL)
    {
        err("There's no point in walking out there -- the action is inside the Data Fortress.");
	return FALSE;
    }

    /* Is the _destination_ anywhere? */
    pRoom = room(x,y,dir);

    if (pRoom == NULL)
    {
        err("There's no point in walking out there -- the action is inside the Data Fortress.");
        return FALSE;
    }
    
    if (pRoom->type == R_GATE)
    {
        if (!PIERCED(pRoom))
        {
            err("You cannot walk into a closed codegate.");
            return FALSE;
        }        
    }

    if (pRoom->type == R_WALL)
    {
        if (!PIERCED(pRoom))
        {
            err("You cannot walk into a solid Data Wall.");
            return FALSE;
        }        
    }
    
    return TRUE;
    
}

bool ice_check(void)
{
    return (this_ice() != NULL);
}


/* Roll for stealth - sneaking past programs. */
bool stealth_check(RUNNER_DATA * pRunner)
{
    int attack, defense;

    attack = stealth_bonus(pRunner) + pRunner->rank + dice(1, 10) - rom_penalty();
    defense = this_ice()->str + server->level + dice(1, 10);
    
    return (attack >= defense);
}


int stealth_bonus(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    
    pProgram = best_stealth(pRunner);
    
    if (pProgram == NULL)
        return 0;
    else
        return (pProgram->str - pProgram->virus);
}

PROGRAM_DATA * best_stealth(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    PROGRAM_DATA * pBest;
    
    pProgram = pRunner->programs;
    pBest = NULL;
    
    while (pProgram != NULL)
    {
        if (pProgram->type == P_STEALTH)
        {
            if (pProgram->duration > 0)
            {
                if (pProgram->str > (pBest == NULL ? 0 : pBest->str))
                {
                    pBest = pProgram;
                }
            }
        }                
        
        pProgram = pProgram->next;
    }
    
    return pBest;        
}












