#include <netrun.h>

/*
 * The server-server netspace (city grid) routines. 
 */

void do_netspace(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;

    /* Check for glue.
    
    if (pRunner->glue)
    {
        desc("You wait around until the glue wears off, all the time wondering how you got out of there.\n");
        pRunner->glue = 0;
    }
    
    */

    repeat = TRUE;
    while (repeat)
    {
        if (where != W_NETSPACE)
        {
            if (where != W_SERVER)
                running_off();
                
            return;
        }
        
        running_on();
        display_running(pRunner);        
                 
        choice = input("Now what?");
        
        if (!strcmp(choice,"R"))           
            do_running_runner_run(pRunner); /* No, really. This does check. */
        else if (!strcmp(choice,"C"))
            err("There's nothing out here worth copying.");            
        else if (!strcmp(choice,"D"))
            do_running_runner_delete(pRunner);
        else if(!strcmp(choice,"N") || !strcmp(choice, "8"))        
            do_netspace_move(pRunner, DIR_N);                
        else if(!strcmp(choice,"S") || !strcmp(choice, "2"))        
            do_netspace_move(pRunner, DIR_S);        
        else if(!strcmp(choice,"E") || !strcmp(choice, "6"))        
            do_netspace_move(pRunner, DIR_E);        
        else if(!strcmp(choice,"W") || !strcmp(choice, "4"))
            do_netspace_move(pRunner, DIR_W); 
        else if(!strcmp(choice,"J"))
            do_netspace_jack(pRunner);
        else
            err("Do what?");        
    }
    
    running_off();
    
    return;
}

NETSPACE_DATA * net_space(int x, int y, int direction)
{
    if ((x < 1 - 1) || (x > net_size_x - 1))
    {
        return NULL;
    }
    if ((y < 1 - 1) || (y > net_size_y - 1))
    {
        return NULL;
    }

    if ((direction < 0) || (direction > MAX_DIR))
    {
        return NULL;
    }

    return net_space_aux(dmx(x,direction),dmy(y,direction));
}

NETSPACE_DATA * net_space_aux(int x, int y)
{
    if ((x < 1 - 1) || (x > net_size_x - 1))
    {
        return NULL;
    }

    if ((y < 1 - 1) || (y > net_size_y - 1))
    {
        return NULL;
    }    
    
    return &netspace[x][y];    
}

void do_netspace_move(RUNNER_DATA * pRunner, int dir)
{
    if (net_space(pRunner->net_x, pRunner->net_y, MAX_DIR) == NULL)
    {
        err("You aren't anywere interesting, either.");
        return;
    }
    
    if (net_space(pRunner->net_x, pRunner->net_y, dir) == NULL)
    {
        err("There isn't anything out there in Deadspace.");
        return;
    }
    
    pRunner->net_x = dmx(pRunner->net_x, dir);
    pRunner->net_y = dmy(pRunner->net_y, dir);
}

void do_netspace_jack(RUNNER_DATA * pRunner)
{
    if (net_space(pRunner->net_x, pRunner->net_y, MAX_DIR) != NULL)
    {
        if (IS_SERVER(net_space(pRunner->net_x, pRunner->net_y, MAX_DIR)->type))
        {
            enter_server(pRunner, net_space(pRunner->net_x, pRunner->net_y, MAX_DIR));
            return;
        }
    }
    
    desc("You jack out.\n");
    stop_run(pRunner);
    where = W_HOME;
    return;
}

void enter_server(RUNNER_DATA * pRunner, NETSPACE_DATA * pNetspace)
{
    int level;

    if (pRunner->meat <= 1)
    {
        if (!yn("You only have one meat point left. Are you sure you want to make a run?"))
        {
            return;
        }
    }

    if (pRunner->rom <= 1)
    {
        if (!yn("Your deck only has one ROM point left. Are you sure you want to make a run?"))
        {
            return;
        }
    }
    
    level = netspace_level(pNetspace);
    
    if (level > pRunner->rank + 2)
    {
        if (!yn("That server is much above your rank. Run against it?"))
        {
            return;
        }
    }
    
    if (pNetspace->pServer == NULL)
    {
        pNetspace->pServer = init_server(level, pNetspace->name);
    }
    
    server = pNetspace->pServer;
    
    where = W_SERVER;
    
    desc("You make your way to the front of %s Data Fortress.\n", server->name);
    morewait();
    
    return;        
}




/*
 * Fill the server-server netspace (City grid).
 *
 * HACK That switch should work, but it's messy-looking. Perhaps a subroutine?
 */
void create_netspace(void)
{
    int x;
    int y;
    int name[(MAX_LEVEL + 1)];
    int i;
    
    for (i = 0; i < (MAX_LEVEL + 1); i++)
    {
        /* Start off with a random name, then cycle through the rest of the list. */
        name[i] = number_range(0, (MAX_NAMES - 1) - 1);
    }

    /* HACK */
    alloc_netspace();
    
    for (x = 0; x < net_size_x - 1; x++)
    {
        for (y = 0; y < net_size_y - 1; y++)
        {
            NETSPACE_DATA * pNetspace;

            pNetspace = NULL;            
            pNetspace = &netspace[x][y];

            /* HACK Sort of. 'y','x'?! */        
            switch(netspace_layout[y][x])
            {  
                case '*':
                    start_net_x = x;
                    start_net_y = y;
                default:
                    pNetspace->type = N_NONE;
                    pNetspace->name = NULL;
                    break;
                case '|':
                    pNetspace->type = N_NS;
                    pNetspace->name = NULL;
                    break;
                case '-': 
                    pNetspace->type = N_EW;
                    pNetspace->name = NULL;
                    break;
                case '+': 
                    pNetspace->type = N_TEE;
                    pNetspace->name = NULL;
                    break;
                case '1':
                    pNetspace->type = N_LEVEL_1;
                    pNetspace->name = (char*) server_names[netspace_level(pNetspace) - 1][name[netspace_level(pNetspace)]];
                    name[netspace_level(pNetspace)]++;
                    if (name[netspace_level(pNetspace)] >= (MAX_NAMES - 1)) 
                        name[netspace_level(pNetspace)] = 0;
                    break;
                case '2':
                    pNetspace->type = N_LEVEL_2;
                    pNetspace->name =(char*) server_names[netspace_level(pNetspace) - 1][name[netspace_level(pNetspace)]];
                    name[netspace_level(pNetspace)]++;
                    if (name[netspace_level(pNetspace)] >= (MAX_NAMES - 1)) 
                        name[netspace_level(pNetspace)] = 0;
                    break;
                case '3':
                    pNetspace->type = N_LEVEL_3;
                    pNetspace->name = (char*)server_names[netspace_level(pNetspace) - 1][name[netspace_level(pNetspace)]];
                    name[netspace_level(pNetspace)]++;
                    if (name[netspace_level(pNetspace)] >= (MAX_NAMES - 1)) 
                        name[netspace_level(pNetspace)] = 0;
                    break;
                case '4':
                    pNetspace->type = N_LEVEL_4;
                    pNetspace->name = (char*)server_names[netspace_level(pNetspace) - 1][name[netspace_level(pNetspace)]];
                    name[netspace_level(pNetspace)]++;
                    if (name[netspace_level(pNetspace)] >= (MAX_NAMES - 1)) 
                        name[netspace_level(pNetspace)] = 0;
                    break;
                case '5':
                    pNetspace->type = N_LEVEL_5;
                    pNetspace->name = (char*)server_names[netspace_level(pNetspace) - 1][name[netspace_level(pNetspace)]];
                    name[netspace_level(pNetspace)]++;
                    if (name[netspace_level(pNetspace)] >= (MAX_NAMES - 1)) 
                        name[netspace_level(pNetspace)] = 0;
                    break;                    
            }
            
            pNetspace->pServer = NULL;
            
            if (IS_SERVER(pNetspace->type) && pNetspace->name == NULL)
            {
                pNetspace->name = "!Bug";
                bug("init_netspace: netspace X: %d Y: %d lv %d no name", x, y, netspace_level(pNetspace));
            }

        }
    }
}

/*
 * A quick HACK to convert netspace types to integer levels.
 */
int netspace_level(NETSPACE_DATA * pNetspace)
{
    int level;

    switch(pNetspace->type)
    {
        default:
        case N_LEVEL_1:
            level = 1;
            break;
        case N_LEVEL_2:
            level = 2;
            break;
        case N_LEVEL_3:
            level = 3;
            break;
        case N_LEVEL_4:
            level = 4;
            break;
        case N_LEVEL_5:
            level = 5;
            break;
    }
    
    return level;
}

/*
 * Allocates the netspace grid (of pointers).
 */
 
void alloc_netspace(void)
{
    int i;

    netspace = calloc_mem(net_size_x, sizeof (NETSPACE_DATA * ));
    
    for (i = 0; i < net_size_y; i++)
    { 
        netspace[i] = calloc_mem(net_size_y, sizeof(NETSPACE_DATA));
    }
}
