#include "netrun.h"

/*
 * Program master location, and creation. This code is hairy. (Someone got a shaver?)
 */ 

/* Locate a program by type and strength. As different types of programs are added (a STR 2 anti-p that stuns, kills, or viruses?), this function may have to take extra args... */
PROGRAM_MASTER * find_prog(program_full_type fulltype, int str)
{
    int i;
    
    i = 0;
    
    while(program_list[i].name != NULL)
    {
        if (program_list[i].fulltype == fulltype)
        {
            if (program_list[i].str == str)
            {
                return (PROGRAM_MASTER *) &program_list[i];
            }
        }
        
        i++;
    }
            
    return NULL;
}
 
PROGRAM_DATA * create_program(PROGRAM_MASTER * pMaster)
{
    PROGRAM_DATA * pProgram;
    
    if (pMaster == NULL)
    {
        bug("create_program: uh-oh, null arg...");
        return NULL;
    }
    
    pProgram = new_program_data();
    
    pProgram->name = pMaster->name;
    pProgram->action = pMaster->action;
    pProgram->thr = get_type(pMaster->fulltype)->thr;
    pProgram->fulltype = pMaster->fulltype;
    pProgram->type = get_type(pMaster->fulltype)->type;
    pProgram->str = pMaster->str;
    pProgram->max_str = pMaster->str;
    pProgram->cost = pMaster->cost;
    pProgram->duration = 0;
    pProgram->virus = 0;
    pProgram->next = NULL;
    
    return pProgram;
}


/* HACK Urrgh. I don't like this function. */
PROGRAM_TYPE_LIST * get_type(program_full_type fulltype)
{
    int i;
    
    i = 0;
    
    while(program_type_list[i].run != NULL)
    {
        if (program_type_list[i].fulltype == fulltype)
        {
            return (PROGRAM_TYPE_LIST *)&program_type_list[i];
        }

        i++;
    }
    
    return NULL;
}

/*
 * Misc program-related routines.
 */

bool strength_check(PROGRAM_DATA * pProgram)
{
    if(pProgram->str <= 0)
    {
        err("Your %s is too damaged to be run.",
            pProgram->name);
        return FALSE;
    }
    
    return TRUE;
}

/*
 * Count the number of stored programs.
 */

int num_stored_programs(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram = NULL;
    int i;

    i = 0;

    pProgram = pRunner->stored;

    while (pProgram != NULL)
    {
        i++;
        pProgram = pProgram->next;
    }    
    
    return i;
}

/*
 * Total MU size of programs in deck
 */

int size_programs(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram = NULL;
    int i,j;
    
    i = 0;

    pProgram = pRunner->programs;

    while ( pProgram != NULL )
    {
        for(j = 0; j < program_size(pProgram); j++)
            i++;

        pProgram = pProgram->next;
    }    
 
    return i;   
}

int program_size(PROGRAM_DATA * pProgram)
{
    if (DATA(pProgram))
        return 1;
    else
        return program_size_table[(pProgram->max_str) - 1];
}


/*
 * Moving programs around
 */

/*
 * Remove a program from runner's lists and last_run, so it may be
 * deallocated or put into a list at the programmer's whim.  
 */

/*
 * HACK UNFIN The pointer logic in this is _really_ bad...help!
 */
void remove_program(RUNNER_DATA * pRunner, PROGRAM_DATA * pRemove)
{
    PROGRAM_DATA * pProgram;
    
    if (pRemove == NULL)
    {
        bug("remove_program: uh-oh, null arg...");
        return;
    }

    /* Nope, it's not supposed to be running. */
    if (pRunner->last_run == pRemove)
        pRunner->last_run = NULL;

    /* Stop the program. */
    pRemove->duration = 0;

    /* I hope these two loops work, otherwise I'm in for a world of trouble. */    

    /* Remove from deck */
    /* HACK */
    if (pRunner->programs == pRemove)
    {
        pRunner->programs = pRemove->next;
    }
    else
    {
        pProgram = pRunner->programs;

        while ( pProgram != NULL )
        {
            if (pProgram->next == pRemove)
                break;
            else /* paranoid */
                pProgram = pProgram->next;
        }

        if (pProgram != NULL)
        {
            pProgram->next = pRemove->next;
        }    
    }

    /* ...and remove from storage. */
    /* HACK */    
    if (pRunner->stored == pRemove)
    {
        pRunner->stored = pRemove->next;
    }
    else
    {
        pProgram = pRunner->stored;
        
        while ( pProgram != NULL )
        {
            if (pProgram->next == pRemove)
                break;
            else /* paranoid */    
                pProgram = pProgram->next;
        }

        if (pProgram != NULL)
        {
            pProgram->next = pRemove->next;
        }
    }
    
    pRemove->next = NULL;            
}

/* Put a program on the end of the deck list. */
void program_to_deck(RUNNER_DATA * pRunner, PROGRAM_DATA * pAdd)
{
    PROGRAM_DATA * pProgram;

    if (pAdd == NULL)
    {
        bug("program_to_deck: null arg");
        return;
    }

    /* Paranoid. */    
    pAdd->duration = 0;
    pAdd->next = NULL;

    if (pRunner->programs == NULL)
        pRunner->programs = pAdd;
    else
    {
        pProgram = pRunner->programs;
        
        while(pProgram->next != NULL)
            pProgram = pProgram->next;    
     
        pProgram->next = pAdd;
    }

}

/* Put a program on the end of the stored list. */
void program_to_stored(RUNNER_DATA * pRunner, PROGRAM_DATA * pAdd)
{
    PROGRAM_DATA * pProgram;

    if (pAdd == NULL)
    {
        bug("program_to_stored: null arg");
        return;
    }

    /* Paranoid. */    
    pAdd->duration = 0;
    pAdd->next = NULL;

    if (pRunner->stored == NULL)
        pRunner->stored = pAdd;
    else
    {    
        pProgram = pRunner->stored;
    
        while(pProgram->next != NULL)
            pProgram = pProgram->next;    
 
        pProgram->next = pAdd;
    }
}



/*
 * The functions called when various types of programs are attempted to run. 
 * Some of these just return err()s, others are just placeholders for stuff
 * used in other parts of the code, e.g armor and stealth programs. But mostly,
 * the 'work' is done here.
 *
 * Incidentally, the type, arguments, etc of these functions are slightly hard to change. They also shouldn't _need_ to be.
 */

void program_anti(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    int attack, defense;
    
    if(this_ice() == NULL)
    {
        err("There aren't any programs to attack here.");
        return;
    }

    desc("You run %s. ", pProgram->name);
    
    desc(pProgram->action,
        this_ice()->name);
    
    attack = (pProgram->str  - pProgram->virus) + pRunner->rank + dice(1,10) - rom_penalty();
    defense = this_ice()->str + server->level + dice(1,10);
    
    if (attack >= defense)
    {
        int dam;
        
        dam = dice(1,6);

        if ((this_ice()->str - dam) < 1)
        {
            desc("%s derezzes.\n",
                this_ice()->name);

            /* UNFIN This should be a macro, free_ice_data will complain, _something's_ wrong with this... */
            free_ice_data(this_ice());
            this_room()->ice = NULL;             
        }
        else
        {
            this_ice()->str -= dam;
            desc("The %s is now STR %d/%d.\n",
                this_ice()->str,
                this_ice()->max_str);
        }
    }
    else
    {
        desc("The %s remains unharmed.\n",
            this_ice()->name);
    }
    
    pProgram->duration = 0;

    /* HACK */
    if (this_ice() == NULL)
        morewait();
    set_next(ST_SERVER);       
}

void program_intr(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    int attack, defense;
    int direction;
    bool repeat;

    repeat = TRUE;
    while(repeat)
    {
        direction = input_direction("Run %s", pProgram->name);

        if (direction == MAX_DIR)
        {
            return;
        }            
        else if ((room(runner_loc(),direction) == NULL) || (room(runner_loc(),direction)->type != R_WALL))
        {
            err("There's nothing there to run an intrusion program on.");           
        }        
        else if (PIERCED(room(runner_loc(),direction)))
        {
            err("That Data Wall has already been broken open.");
        }
        else
            repeat = FALSE;
    }
        
    attack = (pProgram->str - pProgram->virus) + pRunner->rank + dice(1,10) - rom_penalty();
    defense = (server->level * 2) + dice(1,10);

    desc("You run %s. ", pProgram->name);
    desc(pProgram->action, dir_name_table[direction]);
    if (this_ice() == NULL)
        desc("\n");
    
    if ((attack >= defense) && !TRIED(room(runner_loc(),direction)))
    {
        desc("A hole appears in the Data Wall.\n");
        /* HACK should use a macro for setting this */
        SET_BIT(room(runner_loc(),direction)->flags,ROOM_OPEN);
    }
    else
    {
        desc("The Data Wall remains unharmed.\n");        
    }

    SET_BIT(room(runner_loc(),direction)->flags,ROOM_TRIED);

    pProgram->duration = 0;

    /* HACK */
    if (this_ice() == NULL)
        morewait();
    set_next(ST_SERVER);
}

void program_decr(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    int attack, defense;
    int direction;
    bool repeat;

    repeat = TRUE;
    while(repeat)
    {
        direction = input_direction("Run %s", pProgram->name);

        if (direction == MAX_DIR)
        {
            return;
        }            
        else if ((room(runner_loc(),direction) == NULL) || (room(runner_loc(),direction)->type != R_GATE))
        {
            err("There's nothing there to run a decryption program on.");           
        }        
        else if (PIERCED(room(runner_loc(),direction)))
        {
            err("That codegate has already been decrypted open.");
        }
        else 
            repeat = FALSE;
    }
        
    attack = (pProgram->str - pProgram->virus) + pRunner->rank + dice(1,10) - rom_penalty();
    defense = (server->level * 2) + dice(1,10);

    desc("You run %s. ", pProgram->name);
    desc(pProgram->action, dir_name_table[direction]);
    if (this_ice() == NULL)
        desc("\n");

    if ((attack >= defense) && !TRIED(room(runner_loc(),direction)))
    {
        desc("The codegate opens.\n");
        /* HACK should use a macro for setting this */
        SET_BIT(room(runner_loc(),direction)->flags,ROOM_OPEN);
    }
    else
    {
        desc("The codegate remains closed.\n");
    }

    SET_BIT(room(runner_loc(),direction)->flags,ROOM_TRIED);

    pProgram->duration = 0;

    /* HACK */
    if (this_ice() == NULL)
        morewait();
    set_next(ST_SERVER);
}

void program_prot(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    desc("You run %s. %s\n", 
        pProgram->name,
        pProgram->action);

    pProgram->duration = 10;

    /* HACK */
    if (this_ice() == NULL)
        morewait();
    set_next(ST_SERVER);
}

void program_stealth(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    desc("You run %s. %s\n", 
        pProgram->name,
        pProgram->action);
        
    pProgram->duration = 10;

    /* HACK */
    if (this_ice() == NULL)
        morewait();
    set_next(ST_SERVER);
}

void program_info(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    err("%s is not a program.", pProgram->name);
}

void program_cred(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    err("%s is not a program.", pProgram->name);
}

/* UNFIN Make this use a type name. */
void program_no_use_netspace(RUNNER_DATA * pRunner, PROGRAM_DATA * pProgram)
{
    err("There is no use for %s here.", pProgram->name);
}



 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
