#include "netrun.h"

void stop_run(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    
    /* Halt running programs. */
    
    pProgram = pRunner->programs;
    
    while (pProgram != NULL)
    {
        if (pProgram->duration > 0)
            pProgram->duration = 0;
        
        pProgram = pProgram->next;
    }

    morewait();

    running_off();

    if (pRunner->meat != 0)    
        util_repair(pRunner);

    log("Left net.");

    set_next(ST_DONE);
}

/*************************/
/* Main netrunning loop. */
/*************************/

void do_running(RUNNER_DATA * pRunner)
{
    bool repeat;

    log("Entered server.");

    /* Init server run. */     
    server_x = server->start_x;
    server_y = server->start_y;
    set_next(ST_RUNNER);    
    
    /* Show inital situation. */
    display_running(pRunner);
    
    repeat = TRUE;

    while(repeat)
    {
        if (where != W_SERVER)
            return;
    
        running_on();
                    
        switch(turn_next())
        {
            default:
                bug("Odd next_turn");
            case ST_RUNNER:
                do_running_runner(pRunner);
                break;
            case ST_SERVER:    
                do_ice(pRunner);
                break;
            case ST_NEXT:
                do_running_next(pRunner);
                break;    
            case ST_DONE:
                repeat = FALSE;
                break;    
        }
    }

    return;
}

/***************/
/* Server turn */
/***************/

/*
 * See ice.c for the programs subs
 */

void do_ice(RUNNER_DATA * pRunner)
{
    if (where != W_SERVER)
        return;
    
    if (turn_next() != ST_SERVER)
        return;
 
    if (this_ice() == NULL)
    {            
        set_next(ST_NEXT);
        return;
    }
    
    /* HACK Is it? */
    if (this_ice()->str > 0)    
        get_ice_type(this_ice()->fulltype)->run(pRunner, this_ice());
}

/*****************/
/* Runner's turn */
/*****************/

/*
 * Runner's turn: main menu.
 */

void do_running_runner(RUNNER_DATA * pRunner)
{
    bool repeat;
    char * choice;
    
    /* Check for glue.
    
    if (pRunner->glue)
    {
        desc("You can't act!\n");
        set_next(ST_SERVER);
    }
    
    */

    if (where != W_SERVER)
        return;
    
    if (turn_next() != ST_RUNNER)
        return;
    
    /* HACK */
    pRunner->last_run = NULL;
    
    repeat = TRUE;
    while (repeat)
    {
        repeat = FALSE;
        
        /* Menu printed elsewhere. */

        choice = input("Do what?");
        
        if(!strcmp(choice,"R"))
            do_running_runner_run(pRunner);
        else if(!strcmp(choice,"C"))
            do_running_runner_copy(pRunner);
        else if(!strcmp(choice,"D"))
            do_running_runner_delete(pRunner);
        else if(!strcmp(choice,"N") || !strcmp(choice, "8"))        
            do_running_move(pRunner, DIR_N);                
        else if(!strcmp(choice,"S") || !strcmp(choice, "2"))        
            do_running_move(pRunner, DIR_S);        
        else if(!strcmp(choice,"E") || !strcmp(choice, "6"))        
            do_running_move(pRunner, DIR_E);        
        else if(!strcmp(choice,"W") || !strcmp(choice, "4"))
            do_running_move(pRunner, DIR_W);        
        else if(!strcmp(choice,"J"))
            do_running_runner_jack(pRunner);
        else
        {
            err("Do what?");
            repeat = TRUE;
        }
    }
}

/*
 * Run a program.
 * Nice name for the function...whew!
 */
 
void do_running_runner_run(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    bool repeat;

    if (where != W_SERVER && where != W_NETSPACE)
        return;
        
    if(pRunner->programs == NULL)
    {
        err("You have no programs to run!");
        return;
    }
    
    repeat = TRUE;
    while(repeat)
    {
        repeat = FALSE;
        
        pProgram = input_deck_program(pRunner, "Run which program?");
        
        if (pProgram == NULL)
        {
            return;
        }
        
        /* HACK (for anti-p ice) */        
        pRunner->last_run = pProgram;

        if (strength_check(pProgram))
        {
            if (where == W_SERVER)
                get_type(pProgram->fulltype)->run(pRunner, pProgram);
            else if (where == W_NETSPACE)
                get_type(pProgram->fulltype)->run_netspace(pRunner, pProgram);                
        }
    }    
}

void do_running_runner_copy(RUNNER_DATA * pRunner)
{
    if (where != W_SERVER)
        return;
    
    /* HACK flags directly accessed */
    if (!IS_SET(this_room()->flags,ROOM_CASH) && !IS_SET(this_room()->flags,ROOM_INFO))
    {        
        err("There aren't any 'interesting' files here to copy.");
        return;
    }

    /* HACK hard-coded file-size (check) */
    if(size_programs(pRunner) + 1 > MAX_MU)
    {
        err("You don't have enough room in your deck to hold more information.");
        return;
    }
    
    if (ice_check())
    {
        if (stealth_bonus(pRunner))
        {
            if (stealth_check(pRunner))
            {
                desc("You successfully sneak past the %s and get at the file. ",
                    this_ice()->name);
            }
            else
            {
                desc("You fail to sneak past the %s and get at the file. Oops.\n",
                    this_ice()->name);
                set_next(ST_SERVER);
                return;
            }
        }
        else
        {
            err("You can't copy the file with the %s watching you.\n",
                this_ice()->name);
            return;
        }
    }

    desc("The file is ");

    /* HACK flags directly accessed */    
    if(IS_SET(this_room()->flags, ROOM_CASH))
    {
        desc("an unsecured cheque file.");
        program_to_deck(pRunner, create_program(find_prog(P_CRED, server->level)));
        /* HACK macros like this aren't supposed to be used for modifying the struct! */
        REMOVE_BIT(this_room()->flags, ROOM_CASH);
    }
    /* HACK flags directly accessed */
    else if (IS_SET(this_room()->flags, ROOM_INFO))
    {
        desc("interesting-looking.");
	program_to_deck(pRunner, create_program(find_prog(P_INFO, server->level)));
        REMOVE_BIT(this_room()->flags, ROOM_INFO);
    }
    
    desc(" You `copy' it.\n");
    
    set_next(ST_SERVER);
}    

void do_running_runner_delete(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    
    pProgram = input_deck_program(pRunner, "Delete which program?");
    
    if (pProgram == NULL)
        return;

    desc("You delete %s.\n",
        pProgram->name);
    
    kill_program(pProgram);
    
    set_next(ST_SERVER);
}

void do_running_runner_jack(RUNNER_DATA * pRunner)
{
    if (where != W_SERVER)
        return;
    
    if (ice_check() && (this_room()->type != R_OUT))
    {
        /* The lack of a stealth_bonus() check _is_ deliberate. Without it, it is quite easy for the game to stick. */
        {    
            if (stealth_check(pRunner))
            {
                desc("You manage to get out of sight of the %s. ",
                    this_ice()->name);
            }
            else
            {
                desc("You try to jack out, but the %s sees you. Oops.\n",
                    this_ice()->name);
                set_next(ST_SERVER);
                return;            
            }        
        }
    }
    
    desc("You jack out.\n");
    server = NULL;
    where = W_HOME;
    stop_run(pRunner);
}    

void do_running_next(RUNNER_DATA * pRunner)
{
    PROGRAM_DATA * pProgram;
    
    if (where != W_SERVER)
        return;
    
    if (turn_next() != ST_NEXT)
        return;
    
    /*
     * Ending current turn.
     */

//    /*
//    
//    /* Decrement glue (glue stops any action) */
//    
//    if (pRunner->glue > 0)
//    {    
//        pRunner->glue--;
//        
//        if (pRunner->glue == 0)
//        {
//            desc("You can act again!\n");
//        }
//    }
//    
//    /* Decrement cuffs (cuffs stops jacking out) */
//    
//    if (pRunner->cuffs > 0)
//    {    
//        pRunner->cuffs--;
//        
//        if (pRunner->cuffs == 0)
//        {
//            desc("You can now try to jack out.\n");
//        }
//    }
//    
//    /* Decrement chains (chains stops leaving room) */
//    
//    if (pRunner->chains > 0)
//    {    
//        pRunner->chains--;
//        
//        if (pRunner->chains == 0)
//        {
//            desc("You can now try to leave the room.\n");
//        }
//    }
//    
//    */

    /* Take care of program durations. */
    pProgram = pRunner->programs;
    
    while(pProgram != NULL)
    {
        if (pProgram->duration > 0)
        {
            if (pProgram->duration == 1)
            {
                desc("%s stops running.\n",
                    pProgram->name);
            }
        
            pProgram->duration--;    
        }
        else if (pProgram->duration < 0)
        {
            if (pProgram->duration == -1)
            {
                desc("%s can be used again.\n",
                    pProgram->name);
            }
        
            pProgram->duration++;
        }    
        
        pProgram = pProgram->next;
    }
    
    usleep(200000);
    
    /*
     * Starting new turn. 
     */
    
    display_running(pRunner);

    set_next(ST_RUNNER);
}



























