#include "netrun.h"

/* Defines. */
#if defined(KEY)
#undef KEY
#endif

#define KEY( literal, field, value )					\
				if ( !strcmp( word, literal ) )	        \
				{					\
				    field  = value;			\
				    fMatch = TRUE;			\
				    break;				\
				}

/* HACK */
#define MAX_STRING_LENGTH 200

/*
 * Loading and saving saved games -> server states (walls broken, exact ice), programs, runner.
 *
 * TODO Writing/reading servers.
 * TODO Make loading and saving the game work properly - i.e, able to ditch one game and load another correctly within one process.
 */

bool load_game(void)
{
    bool loaded;
    char * filename;
    
    /* HACK Ugly, but I'm in a hurry. I'll write the keystroke-from-string routine later... */
    if (yn("Start a new game?"))
        return FALSE;
    
    filename = input_long("Load which game? [%s]", save_file);
    
    if (!!strcmp(filename, ""))
        strncpy(save_file, filename, 200);

    loaded = read_game(save_file);
    
    if (loaded)
    {
        event("You wake up in your %s, and after making sure your gear remains, head out.", /* TODO Better string for this. */
            house_name_table[runner->rank]); 
    }
    else
    {
        event("You wake up buried in ciruit-filled goo, wondering what went wrong.");        
    }
    
    return loaded;
}

void save_game(void)
{
    char * filename;
    
    filename = input_long("Save game to where? [%s]", save_file);
    
    if (!!strcmp(filename, ""))
        strncpy(save_file, filename, 200);

    write_game(save_file); 
}

/*************************/
/* Game-level read-write */
/*************************/

/*
 * The entire game. Headings. 
 */

void write_game(char * filename)
{
    FILE * fp;
    PROGRAM_DATA * pProgram;
    int x, y;
    
    if (filename == NULL || filename[0] == '\0')
        return;
    
    fp = fopen(filename, "w");
    
    if (fp == NULL)
    {
        bug("write_game: fopen (what's to say?)");
        return;
    }
    
    /* Now we've got a file... */
    fprintf(fp, "*RUNNER\n");
    write_runner(fp);
    
    for (pProgram = runner->programs; pProgram != NULL; pProgram = pProgram->next)
    {
        fprintf(fp, "*PROGRAM\n");
        write_program(pProgram, fp);
    }    
    
    for (pProgram = runner->stored; pProgram != NULL; pProgram = pProgram->next)
    {
        fprintf(fp, "*STORED\n");
        write_program(pProgram, fp);
    }    

    /* Write out da 'Net. */
    write_netspace(fp);
    
    fprintf(fp, "*END\n\n");
    
    fclose(fp);

    log("Wrote game to %s", filename);    
}

bool read_game(char * filename)
{
    bool repeat;
    bool fMatch;
    char * word;
    FILE * fp;
    int space; /* Do we have a netspace? Counts the number of things loaded, or something like that. */
    
    space = 0;
    
    if (filename == NULL || filename[0] == '\0')
        return;
    
    fp = fopen(filename, "r");
    
    if (fp == NULL)
    {
        return FALSE;
    }
        
    repeat = TRUE;
    while(repeat)
    {
        fMatch = FALSE;
        word = feof(fp) ? "END" : fread_hword(fp);
        
        switch(UPPER(word[0]))
        {
            case '#': /* comment */
                fMatch = TRUE;
                fread_to_eol(fp);
                break;
            case 'E':
                if (!strcmp(word, "END"))
                {
                    fMatch = TRUE;
                    fclose(fp);
                    
                    if (!(space > 1))
                    {
                        log("This player doesn't have a netspace. Loading...");
                        /* HACK */
                        create_netspace();
                    }
                    
                    log("Read game from %s", filename);    
                    return TRUE;
                }
                break;
            case 'N':
                if (!strcmp(word, "NETSPACE_TYPE"))
                {
                    fMatch = TRUE;
                    space++;
                    read_netspace_type(fp);
                    break;
                }
                if (!strcmp(word, "NETSPACE_INITIAL"))
                {
                    fMatch = TRUE;
                    space++;
                    read_netspace_initial(fp);
                    break;
                }
                break;
            case 'R':
                if (!strcmp(word, "RUNNER"))
                {
                    fMatch = TRUE;
                    if (runner != NULL)
                        free_runner_data(runner);
                    runner = new_runner_data();    
                    read_runner(runner, fp);
                    break;
                }
                break;
            case 'P':
                if (!strcmp(word, "PROGRAM"))
                {
                    PROGRAM_DATA * pProgram;
                
                    fMatch = TRUE;
                    pProgram = NULL;
                    
                    pProgram = read_program(fp);
                    if (pProgram != NULL)
                        program_to_deck(runner, pProgram);

                    break;
                }
                break;
            case 'S':
                if (!strcmp(word, "STORED"))
                {
                    PROGRAM_DATA * pProgram;
                
                    fMatch = TRUE;
                    pProgram = NULL;
                    
                    pProgram = read_program(fp);
                    if (pProgram != NULL)
                        program_to_stored(runner, pProgram);

                    break;
                }
                break;
        }
        
        if (!fMatch)
        {
            bug("read_runner: no match %s", word);
            fread_to_eol(fp);
        }        
    }
    
    bug("read_saved_game: wrong exit");
    fclose(fp);
    return FALSE;
}

/*
 * Runner
 */

void write_runner(FILE * fp)
{
    if (fp == NULL)
    {
        bug("write_program: null fp");
        return;
    }

    fprintf(fp, "Armor   %d\n", runner->armor);
    fprintf(fp, "Credits %d\n", runner->credits);
    fprintf(fp, "Days    %d\n", runner->days);
    fprintf(fp, "Deposit %d\n", runner->deposit);
    fprintf(fp, "Loan    %d\n", runner->loan);
    fprintf(fp, "Meat    %d\n", runner->meat);
    fprintf(fp, "Name    %s{~\n", runner->name);
    fprintf(fp, "Net_X   %d\n", runner->net_x);
    fprintf(fp, "Net_Y   %d\n", runner->net_y);
    fprintf(fp, "Rank    %d\n", runner->rank);
    fprintf(fp, "Rom     %d\n", runner->rom);
    fprintf(fp, "Utils   %s\n", print_flags(runner->utils));

    fprintf(fp, "END\n\n");    
}

void read_runner(RUNNER_DATA * pRunner, FILE * fp)
{
    bool repeat;
    bool fMatch;
    char * word;

    repeat = TRUE;
    while(repeat)
    {
        fMatch = FALSE;
        word = feof(fp) ? "End" : fread_wordc(fp);
        
        switch(UPPER(word[0]))
        {
            case '#':
                fMatch = TRUE;
                fread_to_eol(fp);
                break;
            case 'A':    
                KEY("ARMOR", pRunner->armor, fread_number(fp));
                break;
            case 'C':    
                KEY("CREDITS", pRunner->credits, fread_number(fp));
                break;
            case 'D':    
                KEY("DAYS", pRunner->days,  fread_number(fp));
                KEY("DEPOSIT", pRunner->deposit,  fread_number(fp));
                break;
            case 'E':
                if (!strcmp( word, "END"))
                {
                    fMatch = TRUE;
                    return;
                }
                break;
            case 'L':    
                KEY("LOAN", pRunner->loan, fread_number(fp));
                break;
            case 'M':    
                KEY("MEAT", pRunner->meat, fread_number(fp)); 
                break;
            case 'N':    
                if (!strcmp(word, "NAME"))
                {
                    fMatch = TRUE;
                    /* I hope this works. */
                    strncpy(pRunner->name, fread_string(fp), sizeof(pRunner->name));
                    break;
                }
                KEY("NET_X", pRunner->net_x, fread_number(fp));
                KEY("NET_Y", pRunner->net_y, fread_number(fp));
                break;
            case 'R':    
                KEY("RANK", pRunner->rank, fread_number(fp));
                KEY("ROM", pRunner->rom, fread_number(fp));
                break;
            case 'U':    
                KEY("UTILS", pRunner->utils, fread_flag(fp));
                break;
        }
        
        if (!fMatch)
        {
            bug("read_runner: no match %s", word);
            fread_to_eol(fp);
        }
    }
}

/*
 * Programs
 */

void write_program(PROGRAM_DATA * pProgram, FILE * fp)
{
    if (fp == NULL)
    {
        bug("write_program: null fp");
        return;
    }

    fprintf(fp, "Fulltype %s{~\n", write_program_full_type(pProgram));
    fprintf(fp, "Str      %d\n", pProgram->str);
    fprintf(fp, "Max_str  %d\n", pProgram->max_str);
    fprintf(fp, "Virus    %d\n", pProgram->virus);

    fprintf(fp, "END\n\n");
}

PROGRAM_DATA * read_program(FILE * fp)
{
    bool repeat;
    bool fMatch;
    char * word;
    /* HACK Program 'details' - must be updated when program rules change. */
    program_full_type fulltype;
    int str;
    int max_str;
    unsigned int virus;

    fulltype = FP_CRED;
    str = 0;
    max_str = 0;
    virus = 0;

    repeat = TRUE;
    while(repeat)
    {
        fMatch = FALSE;
        word = feof(fp) ? "End" : fread_wordc(fp);
        
        switch(UPPER(word[0]))
        {
            case '#':
                fMatch = TRUE;
                fread_to_eol(fp);
                break;
            case 'E':
                if (!strcmp( word, "END"))
                {
                    PROGRAM_DATA * pProgram;
                    fMatch = TRUE;

                    /* Not nessecarily the safest, but... */
                    if (str > MAX_STR)
                    {
                        bug("read_program: this record gave too much strength");
                        str = MAX_STR;
                    }

                    if (max_str > MAX_STR)
                    {
                        bug("read_program: this record gave too much max_strength");
                        max_str = MAX_STR;
                    }
                    
                    /* _Now_ we build the program. Error-check input. */
                    if (str == 0 && max_str == 0)
                    {
                        bug("read_program: this record gave zero strength_s_!");
                        exit_properly();
                    }
                    else if (str == 0)
                    {
                        bug("read_program: this record gave zero strength");
                        str = max_str;
                    }
                    else if (max_str == 0)
                    {
                        bug("read_program: this record gave zero max_str");
                        max_str = str;
                    }

                    pProgram = create_program(find_prog(fulltype, max_str));
                    pProgram->virus = virus;
                    
                    return pProgram;    
                }
                break;
            case 'F':    
                KEY("FULLTYPE", fulltype, fread_program_full_type(fp));
                break;
            case 'M':    
                KEY("MAX_STR", max_str,  fread_number(fp));
                break;
            case 'S':    
                KEY("STR", str, fread_number(fp));
                break;
            case 'V':    
                KEY("VIRUS", virus, fread_number(fp));
                break;
        }
        
        if (!fMatch)
        {
            bug("read_program: no match %s", word);
            fread_to_eol(fp);
        }
    }
    
    bug("read_program: wrong exit");
    return NULL;
}

/*
 * Netspace
 */

void write_netspace(FILE * fp)
{
    int x, y;
    
    fprintf(fp, "*NETSPACE_TYPE\n");
    fprintf(fp, "X %d\n", net_size_x);
    fprintf(fp, "Y %d\n", net_size_y);
    
    for (y = 0; y < net_size_y; y++)
    {
        fprintf(fp, "L |");
        
        for (x = 0; x < net_size_x; x++)
        {
            int type;
            char c;
            
            type = net_space(x, y, MAX_DIR)->type;
            
            if (type == N_NONE)
            {
                c = ' ';
            }
            else
            {
                c = write_netspace_type(net_space(x, y, MAX_DIR)->type);
            }
            
            fprintf(fp, "%c", c);
        }
        
        fprintf(fp, "{~\n");
    }

    fprintf(fp, "END\n\n\n");

    fprintf(fp, "*NETSPACE_INITIAL\n");
    fprintf(fp, "X %d\n", net_size_x);
    fprintf(fp, "Y %d\n", net_size_y);
    
    for (y = 0; y < net_size_y; y++)
    {
        fprintf(fp, "L |");
        
        for (x = 0; x < net_size_x; x++)
        {
            char c;
            NETSPACE_DATA * pNetspace;
            
            pNetspace = net_space(x, y, MAX_DIR);
            
            if (!IS_SERVER(pNetspace->type))
            {
                c = ' ';
            }
            else
            {
                c = pNetspace->name[0];
            }
                        
            fprintf(fp, "%c", c);
        }
        
        fprintf(fp, "{~\n");
    }

    fprintf(fp, "END\n\n");
}

void read_netspace_type(FILE * fp)
{
    TXT_GRID * pGrid;
    int x, y;
    
    pGrid = read_txt_grid(fp); /* Hope this works. */

    if (pGrid == NULL)
    {
        bug("read_netspace_type: read_txt_grid failed");
        return;
    }    
    
    /*
    if (netspace == NULL)
        ....
    */
    
    for (y = 0; y < pGrid->y; y++)
    {
        for (x = 0; x < pGrid->x; x++)
        {
            net_space(x, y, MAX_DIR)->type = read_netspace_type_aux(pGrid->grid[y][x]);
        }
    }
    
    free_txt_grid(pGrid);
}

void read_netspace_initial(FILE * fp)
{
    TXT_GRID * pGrid;
    int x, y;
    
    pGrid = read_txt_grid(fp); /* Hope this works. */

    if (pGrid == NULL)
    {
        bug("read_netspace_initial: read_txt_grid failed");
        return;
    }    
    
    /*
    if (netspace == NULL)
        ....
    */
    
    for (y = 0; y < pGrid->y; y++)
    {
        for (x = 0; x < pGrid->x; x++)
        {
            char initial;
            NETSPACE_DATA * pNetspace;

            initial = pGrid->grid[y][x];
            pNetspace = net_space(x, y, MAX_DIR);
            
            if (pNetspace == NULL)
            {
                bug("read_netspace_type_initial: null netspace X %d Y %d", x, y);
                /*HACK*/
            }
        
            if (initial == ' ')
            {
                pNetspace->name = NULL;                                
            }
            else
            {
                char * name;
            
                name = find_initial(initial, netspace_level(pNetspace));
                
                if (name == NULL)
                {
                    bug("read_netspace: bad initial '%c' X %d Y %d", initial, x, y);
                }

                pNetspace->name = name;                
            }
        }
    }
    
    free_txt_grid(pGrid);
}

/************************************/
/* Low-level game read/write stuff. */
/************************************/

/*
 * TXT_GRIDs.
 */

/* No write function - that must be made specific to the data written. */

/* This allocates a new grid and the memory - it should be freed. */
TXT_GRID * read_txt_grid(FILE * fp)
{
    int line;
    TXT_GRID * pGrid;
    int i;
    bool repeat;
    bool fMatch;
    char * word;    
    
    line = 0;
    pGrid = new_txt_grid();
    
    repeat = TRUE;
    while(repeat)     
    {
        fMatch = FALSE;
        word = feof(fp) ? "End" : fread_wordc(fp);
        
        switch(UPPER(word[0]))
        {
            case '#':
                fMatch = TRUE;
                fread_to_eol(fp);
                break;
            case 'E':
                if (!strcmp(word, "END"))
                {
                    /* HACK No integrity checks! */
                    return pGrid;
                    break;
                }
                break;
            case 'L':
            
                if (!strcmp(word, "L"))
                {
                    char * string;
                    fMatch = TRUE;

                    if (line == 0) /* Allocate the pointer array */
                    {
                        pGrid->grid = calloc_mem(pGrid->y, sizeof(char *));
                    }

                    /* Allocate the string. */
                    pGrid->grid[line] = calloc_mem(pGrid->x + 1, sizeof(char));

                    string = fread_string(fp);
                    string++; /* Move the pointer up one - nudge past the |. */
                   
                    /* HACK Plus one for the \0, minus one for the |. */ 
                    strncpy(pGrid->grid[line], string, pGrid->x);
                    
                    line++;
                }
                break;
            case 'X':
                KEY("X", pGrid->x, fread_number(fp));
                break;
            case 'Y':
                KEY("Y", pGrid->y, fread_number(fp));
                break;
        }
        
        if (!fMatch)
        {
            bug("read_txt_grid: no match %s", word);
            fread_to_eol(fp);                                    
        }
    }        
    
    /* HACK No integrity checks! */
    bug("read_txt_grid: wrong exit");
    return pGrid;
}

/*
 * Program fulltypes.
 */

const char * write_program_full_type(PROGRAM_DATA * pProgram)
{
    int i;

    i = 0;
    
    while(program_type_list[i].run != NULL)
    {
        if(program_type_list[i].fulltype == pProgram->fulltype)
        {
            return program_type_list[i].fulltype_name;
        }
        
        i++;
    }
    
    return "BUG";
}

program_full_type fread_program_full_type(FILE * fp)
{
    int i;
    char * name;
    
    i = 0;
    
    name = fread_string(fp);
    
    while(program_type_list[i].run != NULL)
    {
        if (!strcmp(program_type_list[i].fulltype_name, name))
        {
            return program_type_list[i].fulltype;
        }
        
        i++;
    }
}

/*
 * Net space types.
 */

char write_netspace_type(netspace_type type)
{
    int i;
    
    i = 0;
    
    while(netspace_type_list[i].type != N_NONE)
    {
        if (netspace_type_list[i].type == type)
        {
            return netspace_type_list[i].name;
        }
        
        i++;
    }
    
    bug("write_netspace_type: type not in list, %d", type);
    return ' ';
}

netspace_type read_netspace_type_aux(char name)
{
    int i;
    
    i = 0;
    
    while(netspace_type_list[i].type != N_NONE)
    {
        if (netspace_type_list[i].name == name)
        {
            return netspace_type_list[i].type;
        }
        
        i++;
    }
        
    return N_NONE;
}

/*
 * Server names.
 */

char * find_initial(char initial, int level)
{
    int i;
    
    if (level > MAX_LEVEL)
    {
        bug("find_initial: level too high %d", level);
        return NULL;
    }
    
    if (level < 0)
    {
        bug("find_initial: level too low %d", level);
        return NULL;
    }
    
    for(i = 0; i < MAX_NAMES - 1; i++)
    {
        if (server_names[level - 1][i][0] == initial)
        {
            return server_names[level - 1][i];
        }
    }
    
    bug("find_initial: couldn't find name '%c', level %d", initial, level);
    return NULL;
} 

/************************************/
/* Low level file read/write stuff. */
/************************************/

/* 
 * Read a string from a file. Stored in static buffer. 
 */
 
char * fread_string(FILE * fp)
{
    static char buf[MAX_STRING_LENGTH];
    char c;
    int i; /* I find pointing to characters too confusing. */
    
    i = 0;
    
    /* Skip leading blanks. */
    do
    {
        c = fgetc(fp);
    }
    while (isspace(c));
    
    for ( ;; )
    {
        if (i >= (MAX_STRING_LENGTH - 1))
        {
            bug("fread_string: string too long for buffer");
            return buf;
        }
    
        switch(c)
        {
            default:
                buf[i] = c;
                i++;
                break;
            case '{':
            {
                char d;
                /* HACK Of course, the likelyhood of anyone using the string "{~" in their name is low, but it may still come up. To be fixed. */
                d = fgetc(fp);
                if (d == '~')
                {
                    buf[i] = '\0';
                    i++;
                    return buf;
                }
                else
                {
                    ungetc(d, fp);
                    buf[i] = c;
                    i++;
                }
                break;
            }
            case EOF:
                bug("fread_string: EOF?!");
                buf[i] = '\0';
                return buf;                    
                break;
        }
        
        /* Get the next character. */
        c = fgetc(fp);
    }

    return buf;
}


/*
 * Return a bitfield in ABD... format. Static buffer. 
 */
 
char *print_flags(int flag)
{
    int count, pos = 0;
    static char buf[52];

    for (count = 0; count < 32;  count++)
    {
        if (IS_SET(flag,1<<count))
        {
            if (count < 26)
                buf[pos] = 'A' + count;
            else
                buf[pos] = 'a' + (count - 26);
            pos++;
        }
    }

    if (pos == 0)
    {
        buf[pos] = '0';
        pos++;
    }

    buf[pos] = '\0';

    return buf;
}

/* 
 * Read a number from a file. 
 */
 
int fread_number( FILE *fp )
{
    int number;
    bool sign;
    char c;

    do
    {
	c = fgetc( fp );
    }
    while ( isspace(c) );

    number = 0;

    sign   = FALSE;
    if ( c == '+' )
    {
	c = fgetc( fp );
    }
    else if ( c == '-' )
    {
	sign = TRUE;
	c = fgetc( fp );
    }

    if ( !isdigit(c) )
    {
	bug( "Fread_number: bad format.", 0 );
	exit( 1 );
    }

    while ( isdigit(c) )
    {
	number = number * 10 + c - '0';
	c      = fgetc( fp );
    }

    if ( sign )
	number = 0 - number;

    if ( c == '|' )
	number += fread_number( fp );
    else if ( c != ' ' )
	ungetc( c, fp );

    return number;
}

/* 
 * Reads a flag in ABD... format. 
 */
 
long fread_flag( FILE *fp)
{
    int number;
    char c;
    bool negative = FALSE;

    do
    {
	c = fgetc(fp);
    }
    while ( isspace(c));

    if (c == '-')
    {
	negative = TRUE;
	c = fgetc(fp);
    }

    number = 0;

    if (!isdigit(c))
    {
	while (('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z'))
	{
	    number += flag_convert(c);
	    c = fgetc(fp);
	}
    }

    while (isdigit(c))
    {
	number = number * 10 + c - '0';
	c = fgetc(fp);
    }

    if (c == '|')
	number += fread_flag(fp);

    else if  ( c != ' ')
	ungetc(c,fp);

    if (negative)
	return -1 * number;

    return number;
}

/* 
 * Local to fread_flag(). 
 */
 
long flag_convert(char letter )
{
    long bitsum = 0;
    char i;

    if ('A' <= letter && letter <= 'Z') 
    {
	bitsum = 1;
	for (i = letter; i > 'A'; i--)
	    bitsum *= 2;
    }
    else if ('a' <= letter && letter <= 'z')
    {
	bitsum = 67108864; /* 2^26 */
	for (i = letter; i > 'a'; i --)
	    bitsum *= 2;
    }

    return bitsum;
}

/* Read to end of line (for comments). */
void fread_to_eol( FILE *fp )
{
    char c;

    do
    {
	c = fgetc( fp );
    }
    while ( c != '\n' && c != '\r' );

    do
    {
	c = fgetc( fp );
    }
    while ( c == '\n' || c == '\r' );

    ungetc( c, fp );
    return;
}

/* Inefficient hack. */
char *fread_wordc( FILE * fp )
{
    static char word[MAX_STRING_LENGTH];
    int i;
    
    strncpy(word, fread_word(fp), MAX_STRING_LENGTH);
	        
    for (i = 0; i < MAX_STRING_LENGTH; i++)
    {
        word[i] = UPPER(word[i]);
    }
    
    return word;
}

/* Read one word (into static buffer). */
char *fread_word( FILE *fp )
{
    static char word[MAX_STRING_LENGTH];
    char *pword;
    char cEnd;

    do
    {
	cEnd = fgetc( fp );
    }
    while ( isspace( cEnd ) );

    if ( cEnd == '\'' || cEnd == '"' )
    {
	pword   = word;
    }
    else
    {
	word[0] = cEnd;
	pword   = word+1;
	cEnd    = ' ';
    }

    for ( ; pword < word + MAX_STRING_LENGTH; pword++ )
    {
	*pword = fgetc( fp );
	if ( cEnd == ' ' ? isspace(*pword) : *pword == cEnd )
	{
	    if ( cEnd == ' ' )
		ungetc( *pword, fp );
	    *pword = '\0';
	    
	    return word;
	}
    }

    bug( "Fread_word: word too long.", 0 );
    exit( 1 );
    return NULL;
}

/* HACK */
char * fread_hword(FILE * fp)
{
    char c;

    do
    {
	c = fgetc( fp );
    }
    while ( c != '*' );

    return fread_wordc(fp);
}
 
