#include "netrun.h"

/*
 * Returns a room given a co-ordinate pair and a direction.
 */

ROOM_DATA * room(int x, int y, int direction)
{
    if ((x < 1 - 1) || (x > MAX_ROOMS_X - 1))
    {
        return NULL;
    }
    if ((y < 1 - 1) || (y > MAX_ROOMS_Y - 1))
    {
        return NULL;
    }

    if ((direction < 0) || (direction > MAX_DIR))
    {
        return NULL;
    }

    return room_aux(dmx(x,direction),dmy(y,direction));
}

/* Used to 'legally' return a NULL room pointer for the above, when the
   player is on the edge of the map. */
ROOM_DATA * room_aux(int x, int y)
{
    if ((x < 1 - 1) || (x > MAX_ROOMS_X - 1))
    {
        return NULL;
    }

    if ((y < 1 - 1) || (y > MAX_ROOMS_Y - 1))
    {
        return NULL;
    }    
    
    return server->rooms[x][y];
}

/*
 * Initialise a server struct - place rooms, ICE and spoils from a randomly-chosen layout.
 */

SERVER_DATA * init_server(int level, char * name)
{
    int x, y;
    int layout;
    SERVER_DATA * pServer;
    
    layout = number_range(0, MAX_LAYOUTS - 1);

    pServer = new_server_data();
    
    pServer->level = level;
    pServer->name = name;
    
    for (x = 0; x < MAX_ROOMS_X; x++)
    {
        for (y = 0; y < MAX_ROOMS_Y; y++)
        {
            switch(layout_table[layout][y][x])
            {
                case '#':
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_WALL;
                    break;
                case '@':                
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_OUT;
                    pServer->start_x = x;
                    pServer->start_y = y;
                    break;
                case ' ':
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_OUT;
                    break;
                case '.':
                    pServer->rooms[x][y] = new_room_data();                    
                    pServer->rooms[x][y]->type = R_NONE;
                    break;
                case '+':
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_GATE;
                    break;
                case '-':
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_NONE;
                    assign_ice(pServer->rooms[x][y], pServer->level);
                    break;
                case '|':
                    pServer->rooms[x][y] = new_room_data();
                    pServer->rooms[x][y]->type = R_NONE;
                    assign_spoils(pServer->rooms[x][y], pServer->level);
                    break;
                case '*':
                    pServer->rooms[x][y] = new_room_data();                    
                    pServer->rooms[x][y]->type = R_NONE;
                    assign_ice(pServer->rooms[x][y], pServer->level);
                    assign_spoils(pServer->rooms[x][y], pServer->level);
                    break;
                default:
                    pServer->rooms[x][y] = NULL;
                    break;
            }
        }
    }
    
    /* Whew! Done. (I hope.) */

    return pServer;
}

void assign_spoils(ROOM_DATA * pRoom, int level)
{
    if (one_in_(2))
        SET_BIT(pRoom->flags,ROOM_CASH);
    else
        SET_BIT(pRoom->flags,ROOM_INFO);
}

/* UNFIN The type table is only for standard programs - need to expand it for stunners, etc. */
void assign_ice(ROOM_DATA * pRoom, int level)
{
    ice_full_type fulltype;
    int str;
        
    switch(dice(1,6))
    {
        case 1:
        case 2:
        case 3:
            fulltype = FS_ANTI_P;
            break;
        case 4:
        case 5:
            fulltype = FS_ANTI_D;
            break;
        case 6:
            fulltype = FS_ANTI_U;
            break;
    }

    str = level;

    switch(dice(2,6))
    {
        case 2:
            str -= 2;
            break;
        case 3:
        case 4:
            str -= 1;
            break;
        default:
        case 5:
        case 6:
        case 7:
        case 8:
        case 9:
            str += 0;
            break;
        case 10:
        case 11:
            str += 1;
            break;
        case 12:
            str += 2;
            break;
    }

    if (get_ice_type(fulltype)->type == FS_ANTI_U)
        str -= 1;

    if (str < 1)
        return;

    if (str > MAX_STR)
        str = MAX_STR;
        
    pRoom->ice = create_ice(find_ice(fulltype,str));
}        





